package cn.iocoder.yudao.framework.common.enums;

/**
 * 信息来源枚举类
 * 用于标识信息的来源方式，如流程生成或手动添加
 */
public enum InfoSourceEnum {
    /**
     * 流程生成的信息来源
     */
    FLOW("0", "流程生成"),
    /**
     * 手动添加的信息来源
     */
    MANUAL("1", "手动添加"),
    ;

    // 来源标识符
    private final String source;
    // 来源描述
    private final String desc;

    /**
     * 枚举构造函数
     *
     * @param source 来源标识符
     * @param desc 来源描述
     */
    InfoSourceEnum(String source, String desc) {
        this.source = source;
        this.desc = desc;
    }

    /**
     * 获取来源标识符
     *
     * @return 来源标识符
     */
    public String getSource() {
        return source;
    }

    /**
     * 获取来源描述
     *
     * @return 来源描述
     */
    public String getDesc() {
        return desc;
    }
}
