package cn.iocoder.yudao.module.bpm.controller.admin.oa.business;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.business.vo.OaBusinessPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.business.vo.OaBusinessRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.business.vo.OaBusinessSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.service.oa.business.OaBusinessService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 出差流程信息")
@RestController
@RequestMapping("/bpm/oa-business")
@Validated
public class OaBusinessController {

    @Resource
    private OaBusinessService oaBusinessService;

    @PostMapping("/staging")
    @Operation(summary = "暂存出差审批流程信息")
    @ApiOperationSupport(order = 1)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:staging')")
    public CommonResult<Long> stagingOaBusiness(@RequestBody OaBusinessSaveReqVO stagingReqVO) {
        Long oaBusinessId = oaBusinessService.stagingOaBusiness(stagingReqVO);
        return success(oaBusinessId, "暂存成功");
    }

    @PostMapping("/commit")
    @Operation(summary = "提交出差审批流程信息")
    @ApiOperationSupport(order = 2)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:commit')")
    public CommonResult<Long> commitOaBusiness(@Valid @RequestBody OaBusinessSaveReqVO commitReqVO) {
        Long oaBusinessId = oaBusinessService.commitOaBusiness(commitReqVO);
        return success(oaBusinessId, "提交成功");
    }

    @PostMapping("/agree")
    @Operation(summary = "审批同意出差审批流程信息")
    @ApiOperationSupport(order = 3)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:agree')")
    public CommonResult<Long> agreeOaBusiness(@Valid @RequestBody BpmTaskApproveReqVO agreeReqVO) {
        Long result = oaBusinessService.agreeOaBusiness(agreeReqVO);
        return success(result, "审批成功");
    }

    @PostMapping("/disagree")
    @Operation(summary = "驳回出差审批流程信息")
    @ApiOperationSupport(order = 4)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:disagree')")
    public CommonResult<Long> disagreeOaBusiness(@Valid @RequestBody BpmTaskReturnReqVO disagreeReqVO) {
        Long result = oaBusinessService.disagreeOaBusiness(disagreeReqVO);
        return success(result, "驳回成功");
    }

    @PostMapping("/revocation")
    @Operation(summary = "撤回出差审批流程信息")
    @ApiOperationSupport(order = 5)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:revocation')")
    public CommonResult<Long> revocationOaBusiness(@Valid @RequestBody BpmTaskApproveReqVO revocationReqVO) {
        Long result = oaBusinessService.revocationOaBusiness(revocationReqVO);
        return success(result, "撤回成功");
    }

    @PostMapping("/reCommit")
    @Operation(summary = "驳回或撤回后再次提交出差审批流程信息")
    @ApiOperationSupport(order = 6)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:re-commit')")
    public CommonResult<Long> reCommitOaBusiness(@Valid @RequestBody OaBusinessSaveReqVO reCommitReqVO) {
        Long result = oaBusinessService.reCommitOaBusiness(reCommitReqVO);
        return success(result, "再次提交成功");
    }

    @DeleteMapping("/close")
    @Operation(summary = "驳回或撤回后关闭出差审批流程信息")
    @ApiOperationSupport(order = 7)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-entry:close')")
    public CommonResult<Long> closeOaBusiness(@RequestParam("id") Long id) {
        Long result = oaBusinessService.closeOaBusiness(id);

        return success(result, "关闭成功");
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除出差审批流程信息")
    @ApiOperationSupport(order = 8)
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:delete')")
    public CommonResult<Boolean> deleteOaBusiness(@RequestParam("id") Long id) {
        oaBusinessService.deleteOaBusiness(id);
        return success(true, "删除成功");
    }

    @GetMapping("/get")
    @Operation(summary = "根据id获得出差审批流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1")
    @ApiOperationSupport(order = 9)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:query-id')")
    public CommonResult<OaBusinessRespVO> getOaBusiness(@RequestParam("id") Long id) {
        OaBusinessRespVO respVO = oaBusinessService.getOaBusiness(id);
        return success(respVO, "查询对象成功");
    }

    @GetMapping("/getByProcInstId")
    @Operation(summary = "根据流程实例id获得出差审批流程信息")
    @Parameter(name = "procInstId", description = "流程实例id", required = true, example = "9528a78d-457e-11ef-853c-4c034fce6445")
    @ApiOperationSupport(order = 10)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:query-proc-inst-id')")
    public CommonResult<OaBusinessRespVO> getOaBusinessByProcInstId(@RequestParam("procInstId") String procInstId) {
        OaBusinessRespVO respVO = oaBusinessService.getOaBusinessByProcInstId(procInstId);
        return success(respVO, "查询对象成功");
    }

    @GetMapping("/page")
    @Operation(summary = "获得出差审批流程信息分页")
    @ApiOperationSupport(order = 11)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:query-page')")
    public CommonResult<PageResult<OaBusinessRespVO>> getOaBusinessPage(@Valid @ParameterObject OaBusinessPageReqVO pageReqVO) {
        return success(oaBusinessService.getOaBusinessPage(pageReqVO), "查询列表成功");
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出出差审批流程信息 Excel")
    @ApiOperationSupport(order = 12)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-business:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaBusinessExcel(@ParameterObject OaBusinessPageReqVO pageReqVO,
                                      HttpServletResponse response) throws IOException {
        PageResult<OaBusinessRespVO> pageResult = oaBusinessService.exportOaBusinessExcel(pageReqVO);
        List<OaBusinessRespVO> list = pageResult.getList();
        // 导出 Excel
        ExcelUtils.write(response, "出差审批流程信息.xls", "数据", OaBusinessRespVO.class,
                BeanUtils.toBean(list, OaBusinessRespVO.class));
    }

}