package cn.iocoder.yudao.module.bpm.controller.admin.oa.out;

import cn.iocoder.yudao.module.bpm.controller.admin.oa.out.vo.OaOutPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.out.vo.OaOutRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.out.vo.OaOutSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.out.vo.OaOutPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.out.vo.OaOutRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.out.vo.OaOutSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.service.oa.out.OaOutService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.out.OaOutDO;
import cn.iocoder.yudao.module.bpm.service.oa.out.OaOutService;

@Tag(name = "管理后台 - 公务外出流程信息")
@RestController
@RequestMapping("/bpm/oa-out")
@Validated
public class OaOutController {

    @Resource
    private OaOutService oaOutService;

    @PostMapping("/staging")
    @Operation(summary = "暂存公务外出审批流程信息")
    @ApiOperationSupport(order = 1)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:staging')")
    public CommonResult<Long> stagingOaOut(@RequestBody OaOutSaveReqVO stagingReqVO) {
        Long oaOutId = oaOutService.stagingOaOut(stagingReqVO);
        return success(oaOutId, "暂存成功");
    }

    @PostMapping("/commit")
    @Operation(summary = "提交公务外出审批流程信息")
    @ApiOperationSupport(order = 2)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:commit')")
    public CommonResult<Long> commitOaOut(@Valid @RequestBody OaOutSaveReqVO commitReqVO) {
        Long oaOutId = oaOutService.commitOaOut(commitReqVO);
        return success(oaOutId, "提交成功");
    }

    @PostMapping("/agree")
    @Operation(summary = "审批同意公务外出审批流程信息")
    @ApiOperationSupport(order = 3)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:agree')")
    public CommonResult<Long> agreeOaOut(@Valid @RequestBody BpmTaskApproveReqVO agreeReqVO) {
        Long result = oaOutService.agreeOaOut(agreeReqVO);
        return success(result, "审批成功");
    }

    @PostMapping("/disagree")
    @Operation(summary = "驳回公务外出审批流程信息")
    @ApiOperationSupport(order = 4)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:disagree')")
    public CommonResult<Long> disagreeOaOut(@Valid @RequestBody BpmTaskReturnReqVO disagreeReqVO) {
        Long result = oaOutService.disagreeOaOut(disagreeReqVO);
        return success(result, "驳回成功");
    }

    @PostMapping("/revocation")
    @Operation(summary = "撤回公务外出审批流程信息")
    @ApiOperationSupport(order = 5)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:revocation')")
    public CommonResult<Long> revocationOaOut(@Valid @RequestBody BpmTaskApproveReqVO revocationReqVO) {
        Long result = oaOutService.revocationOaOut(revocationReqVO);
        return success(result, "撤回成功");
    }

    @PostMapping("/reCommit")
    @Operation(summary = "驳回或撤回后再次提交公务外出审批流程信息")
    @ApiOperationSupport(order = 6)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:re-commit')")
    public CommonResult<Long> reCommitOaOut(@Valid @RequestBody OaOutSaveReqVO reCommitReqVO) {
        Long result = oaOutService.reCommitOaOut(reCommitReqVO);
        return success(result, "再次提交成功");
    }

    @DeleteMapping("/close")
    @Operation(summary = "驳回或撤回后关闭公务外出审批流程信息")
    @ApiOperationSupport(order = 7)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-entry:close')")
    public CommonResult<Long> closeOaOut(@RequestParam("id") Long id) {
        Long result = oaOutService.closeOaOut(id);

        return success(result, "关闭成功");
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除公务外出审批流程信息")
    @ApiOperationSupport(order = 8)
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:delete')")
    public CommonResult<Boolean> deleteOaOut(@RequestParam("id") Long id) {
        oaOutService.deleteOaOut(id);
        return success(true, "删除成功");
    }

    @GetMapping("/get")
    @Operation(summary = "根据id获得公务外出审批流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1")
    @ApiOperationSupport(order = 9)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:query-id')")
    public CommonResult<OaOutRespVO> getOaOut(@RequestParam("id") Long id) {
        OaOutRespVO respVO = oaOutService.getOaOut(id);
        return success(respVO, "查询对象成功");
    }

    @GetMapping("/getByProcInstId")
    @Operation(summary = "根据流程实例id获得公务外出审批流程信息")
    @Parameter(name = "procInstId", description = "流程实例id", required = true, example = "9528a78d-457e-11ef-853c-4c034fce6445")
    @ApiOperationSupport(order = 10)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:query-proc-inst-id')")
    public CommonResult<OaOutRespVO> getOaOutByProcInstId(@RequestParam("procInstId") String procInstId) {
        OaOutRespVO respVO = oaOutService.getOaOutByProcInstId(procInstId);
        return success(respVO, "查询对象成功");
    }

    @GetMapping("/page")
    @Operation(summary = "获得公务外出审批流程信息分页")
    @ApiOperationSupport(order = 11)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:query-page')")
    public CommonResult<PageResult<OaOutRespVO>> getOaOutPage(@Valid @ParameterObject OaOutPageReqVO pageReqVO) {
        return success(oaOutService.getOaOutPage(pageReqVO), "查询列表成功");
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出公务外出审批流程信息 Excel")
    @ApiOperationSupport(order = 12)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-out:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaOutExcel(@ParameterObject OaOutPageReqVO pageReqVO,
                                   HttpServletResponse response) throws IOException {
        PageResult<OaOutRespVO> pageResult = oaOutService.exportOaOutExcel(pageReqVO);
        List<OaOutRespVO> list = pageResult.getList();
        // 导出 Excel
        ExcelUtils.write(response, "公务外出审批流程信息.xls", "数据", OaOutRespVO.class,
                BeanUtils.toBean(list, OaOutRespVO.class));
    }

}