package cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp.vo.OaStampPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp.vo.OaStampRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp.vo.OaStampSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.service.oa.stamp.OaStampService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 用印流程信息")
@RestController
@RequestMapping("/bpm/oa-stamp")
@Validated
public class OaStampController {


    @Resource
    private OaStampService oaStampService;

    @PostMapping("/staging")
    @Operation(summary = "暂存用印审批流程信息")
    @ApiOperationSupport(order = 1)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:staging')")
    public CommonResult<Long> stagingOaStamp(@RequestBody OaStampSaveReqVO stagingReqVO) {
        Long oaStampId = oaStampService.stagingOaStamp(stagingReqVO);
        return success(oaStampId, "暂存成功");
    }

    @PostMapping("/commit")
    @Operation(summary = "提交用印审批流程信息")
    @ApiOperationSupport(order = 2)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:commit')")
    public CommonResult<Long> commitOaStamp(@Valid @RequestBody OaStampSaveReqVO commitReqVO) {
        Long oaStampId = oaStampService.commitOaStamp(commitReqVO);
        return success(oaStampId, "提交成功");
    }

    @PostMapping("/agree")
    @Operation(summary = "审批同意用印审批流程信息")
    @ApiOperationSupport(order = 3)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:agree')")
    public CommonResult<Long> agreeOaStamp(@Valid @RequestBody BpmTaskApproveReqVO agreeReqVO) {
        Long result = oaStampService.agreeOaStamp(agreeReqVO);
        return success(result, "审批成功");
    }

    @PostMapping("/disagree")
    @Operation(summary = "驳回用印审批流程信息")
    @ApiOperationSupport(order = 4)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:disagree')")
    public CommonResult<Long> disagreeOaStamp(@Valid @RequestBody BpmTaskReturnReqVO disagreeReqVO) {
        Long result = oaStampService.disagreeOaStamp(disagreeReqVO);
        return success(result, "驳回成功");
    }

    @PostMapping("/revocation")
    @Operation(summary = "撤回用印审批流程信息")
    @ApiOperationSupport(order = 5)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:revocation')")
    public CommonResult<Long> revocationOaStamp(@Valid @RequestBody BpmTaskApproveReqVO revocationReqVO) {
        Long result = oaStampService.revocationOaStamp(revocationReqVO);
        return success(result, "撤回成功");
    }

    @PostMapping("/reCommit")
    @Operation(summary = "驳回或撤回后再次提交用印审批流程信息")
    @ApiOperationSupport(order = 6)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:re-commit')")
    public CommonResult<Long> reCommitOaStamp(@Valid @RequestBody OaStampSaveReqVO reCommitReqVO) {
        Long result = oaStampService.reCommitOaStamp(reCommitReqVO);
        return success(result, "再次提交成功");
    }

    @DeleteMapping("/close")
    @Operation(summary = "驳回或撤回后关闭用印审批流程信息")
    @ApiOperationSupport(order = 7)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-entry:close')")
    public CommonResult<Long> closeOaStamp(@RequestParam("id") Long id) {
        Long result = oaStampService.closeOaStamp(id);

        return success(result, "关闭成功");
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除用印审批流程信息")
    @ApiOperationSupport(order = 8)
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:delete')")
    public CommonResult<Boolean> deleteOaStamp(@RequestParam("id") Long id) {
        oaStampService.deleteOaStamp(id);
        return success(true, "删除成功");
    }

    @GetMapping("/get")
    @Operation(summary = "根据id获得用印审批流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1")
    @ApiOperationSupport(order = 9)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:query-id')")
    public CommonResult<OaStampRespVO> getOaStamp(@RequestParam("id") Long id) {
        OaStampRespVO respVO = oaStampService.getOaStamp(id);
        return success(respVO, "查询对象成功");
    }

    @GetMapping("/getByProcInstId")
    @Operation(summary = "根据流程实例id获得用印审批流程信息")
    @Parameter(name = "procInstId", description = "流程实例id", required = true, example = "9528a78d-457e-11ef-853c-4c034fce6445")
    @ApiOperationSupport(order = 10)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:query-proc-inst-id')")
    public CommonResult<OaStampRespVO> getOaStampByProcInstId(@RequestParam("procInstId") String procInstId) {
        OaStampRespVO respVO = oaStampService.getOaStampByProcInstId(procInstId);
        return success(respVO, "查询对象成功");
    }

    @GetMapping("/page")
    @Operation(summary = "获得用印审批流程信息分页")
    @ApiOperationSupport(order = 11)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:query-page')")
    public CommonResult<PageResult<OaStampRespVO>> getOaStampPage(@Valid @ParameterObject OaStampPageReqVO pageReqVO) {
        return success(oaStampService.getOaStampPage(pageReqVO), "查询列表成功");
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出用印审批流程信息 Excel")
    @ApiOperationSupport(order = 12)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaStampExcel(@ParameterObject OaStampPageReqVO pageReqVO,
                                 HttpServletResponse response) throws IOException {
        PageResult<OaStampRespVO> pageResult = oaStampService.exportOaStampExcel(pageReqVO);
        List<OaStampRespVO> list = pageResult.getList();
        // 导出 Excel
        ExcelUtils.write(response, "用印审批流程信息.xls", "数据", OaStampRespVO.class,
                BeanUtils.toBean(list, OaStampRespVO.class));
    }
}