package cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo.vo.OaStampInfoPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo.vo.OaStampInfoRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo.vo.OaStampInfoSaveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.stamp.stampinfo.OaStampInfoDO;
import cn.iocoder.yudao.module.bpm.service.stamp.stampinfo.OaStampInfoService;
import cn.iocoder.yudao.module.employee.api.EmployeeApi;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.PostApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.dept.dto.PostRespDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 用印信息")
@RestController
@RequestMapping("/bpm/oa-stamp-info")
@Validated
public class OaStampInfoController {

    @Resource
    private OaStampInfoService oaStampInfoService;

    @Resource
    private EmployeeApi employeeApi;

    @Resource
    private DeptApi deptApi;

    @Resource
    private PostApi postApi;

    @Resource
    private FileApi fileApi;

    @PostMapping("/create")
    @Operation(summary = "创建用印信息")
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:create')")
    public CommonResult<Long> createOaStampInfo(@Valid @RequestBody OaStampInfoSaveReqVO createReqVO) {
        return success(oaStampInfoService.createOaStampInfo(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新用印信息")
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:update')")
    public CommonResult<Boolean> updateOaStampInfo(@Valid @RequestBody OaStampInfoSaveReqVO updateReqVO) {
        oaStampInfoService.updateOaStampInfo(updateReqVO);
        return success(true);
    }

    @PutMapping("/invalid")
    @Operation(summary = "作废用印信息")
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:invalid')")
    public CommonResult<Boolean> invalidOaStampInfo(@RequestParam("id") Long id) {
        oaStampInfoService.invalidOaStampInfo(id);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除用印信息")
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:delete')")
    public CommonResult<Boolean> deleteOaStampInfo(@RequestParam("id") Long id) {
        oaStampInfoService.deleteOaStampInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得用印信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:query')")
    public CommonResult<OaStampInfoRespVO> getOaStampInfo(@RequestParam("id") Long id) {
        OaStampInfoDO oaStampInfo = oaStampInfoService.getOaStampInfo(id);
        OaStampInfoRespVO oaStampInfoRespVO = BeanUtils.toBean(oaStampInfo, OaStampInfoRespVO.class);
        if (oaStampInfo != null) {
            // 状态描述
            String status = oaStampInfoRespVO.getStatus();
            switch (status) {
                case "0":
                    oaStampInfoRespVO.setStatusDesc("已完成");
                    break;
                case "1":
                    oaStampInfoRespVO.setStatusDesc("已作废");
                    break;
                default:
                    oaStampInfoRespVO.setStatusDesc("未知");
                    break;
            }
            // 数据来源描述
            String infoSource = oaStampInfoRespVO.getInfoSource();
            switch (infoSource) {
                case "0":
                    oaStampInfoRespVO.setInfoSourceDesc("流程添加");
                    break;
                case "1":
                    oaStampInfoRespVO.setInfoSourceDesc("手动添加");
                    break;
                default:
                    oaStampInfoRespVO.setInfoSourceDesc("未知");
                    break;
            }
            // 创建人名字
            Long createEmployeeId = oaStampInfoRespVO.getCreateEmployeeId();
            EmployeeRespDTO createEmployee = employeeApi.getEmployeeById(createEmployeeId);
            if (createEmployee != null) {
                oaStampInfoRespVO.setCreateEmployeeName(createEmployee.getName());
            }
            // 用印人信息
            Long employeeId = oaStampInfoRespVO.getEmployeeId();
            // AdminUserRespDTO employee = adminUserApi.getUser(employeeId);
            EmployeeRespDTO employee = employeeApi.getEmployeeById(employeeId);
            if (employee != null) {
                oaStampInfoRespVO.setEmployeeName(employee.getName());
                oaStampInfoRespVO.setEmployeePhone(employee.getPhone());
            }
            // 职位
            Long postId = oaStampInfoRespVO.getPostId();
            PostRespDTO post = postApi.getPost(postId);
            if (Objects.nonNull(post)) {
                oaStampInfoRespVO.setPosition(post.getName());
            }
            // 部门名称
            Long deptId = oaStampInfoRespVO.getDeptId();
            DeptRespDTO dept = deptApi.getDept(deptId);
            if (dept != null) {
                oaStampInfoRespVO.setDeptName(dept.getName());
            }

            // 附件
            List<FileDTO> fileDTOListByBiz = fileApi.getFileDTOListByBiz(oaStampInfo.getStampUuid());
            oaStampInfoRespVO.setFileList(fileDTOListByBiz);
            List<Long> fileIdList = fileDTOListByBiz.stream().map(FileDTO::getId).collect(Collectors.toList());
            oaStampInfoRespVO.setFileIdList(fileIdList);
        }


        return success(oaStampInfoRespVO);
    }

    @GetMapping("/page")
    @Operation(summary = "获得用印信息分页")
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:query')")
    public CommonResult<PageResult<OaStampInfoRespVO>> getOaStampInfoPage(@Valid @ParameterObject OaStampInfoPageReqVO pageReqVO) {
        PageResult<OaStampInfoRespVO> pageResult = oaStampInfoService.getOaStampInfoPage(pageReqVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出用印信息 Excel")
    // @PreAuthorize("@ss.hasPermission('bpm:oa-stamp-info:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaStampInfoExcel(@Valid OaStampInfoPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaStampInfoRespVO> list = oaStampInfoService.getOaStampInfoPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "用印信息.xls", "数据", OaStampInfoRespVO.class,
                        list);
    }

}