package cn.iocoder.yudao.module.bpm.dal.mysql.oa.conversion;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion.vo.OaConversionPageReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.conversion.OaConversionDO;
import cn.iocoder.yudao.module.system.api.permission.dto.DeptDataPermissionRespDTO;
import org.apache.ibatis.annotations.Mapper;

/**
 * 转正流程信息 Mapper
 *
 * @author 芋道源码
 */
@Mapper
public interface OaConversionMapper extends BaseMapperX<OaConversionDO> {

    default PageResult<OaConversionDO> selectPage(OaConversionPageReqVO reqVO) {
        LambdaQueryWrapperX<OaConversionDO> lambdaQueryWrapperX = new LambdaQueryWrapperX<OaConversionDO>()
                .eqIfPresent(OaConversionDO::getEmployeeName, reqVO.getEmployeeName())
                .eqIfPresent(OaConversionDO::getDeptId, reqVO.getDeptId())
                .eqIfPresent(OaConversionDO::getPostId, reqVO.getPostId())
                .likeIfPresent(OaConversionDO::getEmployeePhone, reqVO.getEmployeePhone())
                .betweenIfPresent(OaConversionDO::getConversionDate, reqVO.getConversionDate())
                .eqIfPresent(OaConversionDO::getApplyEmployeeName, reqVO.getApplyEmployeeName())
                .betweenIfPresent(OaConversionDO::getCreateTime, reqVO.getCreateTime())
                .eqIfPresent(OaConversionDO::getAuditStatus, reqVO.getAuditStatus())
                .orderByDesc(OaConversionDO::getId);
        // 数据权限
        DeptDataPermissionRespDTO deptDataPermission = reqVO.getDeptDataPermission();
        if (deptDataPermission != null) {
            if (deptDataPermission.getAll()) {
                // 全部数据权限
            } else {
                if (CollUtil.isNotEmpty(deptDataPermission.getDeptIds()) && deptDataPermission.getSelf()) {
                    lambdaQueryWrapperX.and(wrapper ->
                            wrapper.in(OaConversionDO::getDeptId, deptDataPermission.getDeptIds()))
                            .or(wrapper -> wrapper.eq(OaConversionDO::getCreator, reqVO.getUserId()));
                } else if (CollUtil.isNotEmpty(deptDataPermission.getDeptIds())) {
                    lambdaQueryWrapperX.in(OaConversionDO::getCreator, deptDataPermission.getDeptIds());
                } else if (deptDataPermission.getSelf()) {
                    lambdaQueryWrapperX.eq(OaConversionDO::getCreator, reqVO.getUserId());
                } else {
                    return PageResult.empty();
                }
            }
        } else {
            return PageResult.empty();
        }
        return selectPage(reqVO, lambdaQueryWrapperX);

    }

}