package cn.iocoder.yudao.module.bpm.dal.mysql.oa.entry;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntryPageReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.entry.OaEntryDO;
import cn.iocoder.yudao.module.system.api.permission.dto.DeptDataPermissionRespDTO;
import org.apache.ibatis.annotations.Mapper;
/**
 * 入职流程信息 Mapper
 *
 * @author 芋道源码
 */
@Mapper
public interface OaEntryMapper extends BaseMapperX<OaEntryDO> {

    default PageResult<OaEntryDO> selectPage(OaEntryPageReqVO reqVO) {
        LambdaQueryWrapperX<OaEntryDO> lambdaQueryWrapperX = new LambdaQueryWrapperX<OaEntryDO>()
                .likeIfPresent(OaEntryDO::getEntryName, reqVO.getEntryName())
                .eqIfPresent(OaEntryDO::getDeptId, reqVO.getDeptId())
                .eqIfPresent(OaEntryDO::getPostId, reqVO.getPostId())
                .eqIfPresent(OaEntryDO::getEmployeeType, reqVO.getEmployeeType())
                .eqIfPresent(OaEntryDO::getGender, reqVO.getGender())
                .eqIfPresent(OaEntryDO::getContactNumber, reqVO.getContactNumber())
                .betweenIfPresent(OaEntryDO::getCreateTime, reqVO.getCreateTime())
                .betweenIfPresent(OaEntryDO::getEntryDate, reqVO.getEntryDate())
                .eqIfPresent(OaEntryDO::getAuditStatus, reqVO.getAuditStatus())
                .orderByDesc(OaEntryDO::getId);
        // 数据权限
        DeptDataPermissionRespDTO deptDataPermission = reqVO.getDeptDataPermission();
        if (deptDataPermission != null) {
            if (deptDataPermission.getAll()) {
                // 全部数据权限
            } else {
                if (CollUtil.isNotEmpty(deptDataPermission.getDeptIds()) && deptDataPermission.getSelf()) {
                    lambdaQueryWrapperX.and(wrapper ->
                            wrapper.in(OaEntryDO::getDeptId, deptDataPermission.getDeptIds()))
                            .or(wrapper -> wrapper.eq(OaEntryDO::getCreator, reqVO.getUserId()));
                } else if (CollUtil.isNotEmpty(deptDataPermission.getDeptIds())) {
                    lambdaQueryWrapperX.in(OaEntryDO::getCreator, deptDataPermission.getDeptIds());
                } else if (deptDataPermission.getSelf()) {
                    lambdaQueryWrapperX.eq(OaEntryDO::getCreator, reqVO.getUserId());
                } else {
                    return PageResult.empty();
                }
            }
        } else {
            return PageResult.empty();
        }
        return selectPage(reqVO, lambdaQueryWrapperX);

    }

}