package cn.iocoder.yudao.module.bpm.dal.mysql.oa.turnover;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.turnover.vo.OaTurnoverPageReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.turnover.OaTurnoverDO;
import cn.iocoder.yudao.module.system.api.permission.dto.DeptDataPermissionRespDTO;
import org.apache.ibatis.annotations.Mapper;

/**
 * 离职流程信息 Mapper
 *
 * @author dp
 */
@Mapper
public interface OaTurnOverMapper extends BaseMapperX<OaTurnoverDO> {

    default PageResult<OaTurnoverDO> selectPage(OaTurnoverPageReqVO reqVO) {
        LambdaQueryWrapperX<OaTurnoverDO> lambdaQueryWrapperX = new LambdaQueryWrapperX<OaTurnoverDO>()
                .likeIfPresent(OaTurnoverDO::getEmployeeName, reqVO.getEmployeeName())
                .likeIfPresent(OaTurnoverDO::getEmployeePhone, reqVO.getEmployeePhone())
                .eqIfPresent(OaTurnoverDO::getDeptId, reqVO.getDeptId())
                .eqIfPresent(OaTurnoverDO::getPostId, reqVO.getPostId())
                .betweenIfPresent(OaTurnoverDO::getTurnoverDate, reqVO.getTurnoverDate())
                .eqIfPresent(OaTurnoverDO::getAuditStatus, reqVO.getAuditStatus())
                .likeIfPresent(OaTurnoverDO::getApplyEmployeeName, reqVO.getApplyEmployeeName())
                .betweenIfPresent(OaTurnoverDO::getCreateTime, reqVO.getCreateTime())
                .orderByDesc(OaTurnoverDO::getId);
        // 数据权限
        DeptDataPermissionRespDTO deptDataPermission = reqVO.getDeptDataPermission();
        if (deptDataPermission != null) {
            if (deptDataPermission.getAll()) {
                // 全部数据权限
            } else {
                if (CollUtil.isNotEmpty(deptDataPermission.getDeptIds()) && deptDataPermission.getSelf()) {
                    lambdaQueryWrapperX.and(wrapper ->
                            wrapper.in(OaTurnoverDO::getDeptId, deptDataPermission.getDeptIds()))
                            .or(wrapper -> wrapper.eq(OaTurnoverDO::getCreator, reqVO.getUserId()));
                } else if (CollUtil.isNotEmpty(deptDataPermission.getDeptIds())) {
                    lambdaQueryWrapperX.in(OaTurnoverDO::getCreator, deptDataPermission.getDeptIds());
                } else if (deptDataPermission.getSelf()) {
                    lambdaQueryWrapperX.eq(OaTurnoverDO::getCreator, reqVO.getUserId());
                } else {
                    return PageResult.empty();
                }
            }
        } else {
            return PageResult.empty();
        }
        return selectPage(reqVO, lambdaQueryWrapperX);

    }

}