package cn.iocoder.yudao.module.bpm.dal.mysql.task;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.QueryWrapperX;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import cn.iocoder.yudao.module.bpm.dal.dataobject.definition.BpmTaskAssignRuleDO;
import org.apache.ibatis.annotations.Mapper;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.*;
import org.springframework.lang.Nullable;

/**
 * Bpm 任务规则 Mapper
 *
 * @author 芋道源码
 */
@Mapper
public interface BpmTaskAssignRuleMapper extends BaseMapperX<BpmTaskAssignRuleDO> {

    default PageResult<BpmTaskAssignRuleDO> selectPage(BpmTaskAssignRulePageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<BpmTaskAssignRuleDO>()
                .eqIfPresent(BpmTaskAssignRuleDO::getModelId, reqVO.getModelId())
                .eqIfPresent(BpmTaskAssignRuleDO::getProcessDefinitionId, reqVO.getProcessDefinitionId())
                .eqIfPresent(BpmTaskAssignRuleDO::getTaskDefinitionKey, reqVO.getTaskDefinitionKey())
                .eqIfPresent(BpmTaskAssignRuleDO::getType, reqVO.getType())
                .eqIfPresent(BpmTaskAssignRuleDO::getOptions, reqVO.getOptions())
                .betweenIfPresent(BpmTaskAssignRuleDO::getCreateTime, reqVO.getCreateTime())
                .orderByDesc(BpmTaskAssignRuleDO::getId));
    }

    default List<BpmTaskAssignRuleDO> selectListByModelId(String modelId) {
        return selectList(new QueryWrapperX<BpmTaskAssignRuleDO>()
                .eq("model_id", modelId)
                .eq("process_definition_id", BpmTaskAssignRuleDO.PROCESS_DEFINITION_ID_NULL));
    }

    @TenantIgnore
    default List<BpmTaskAssignRuleDO> selectListByModelId2(String modelId) {
        return selectList(new QueryWrapperX<BpmTaskAssignRuleDO>()
                .eq("tenant_id", 1L)
                .eq("model_id", modelId)
                .eq("process_definition_id", BpmTaskAssignRuleDO.PROCESS_DEFINITION_ID_NULL));
    }

    default List<BpmTaskAssignRuleDO> selectListByProcessDefinitionId(String processDefinitionId,
                                                                      @Nullable String taskDefinitionKey) {
        return selectList(new QueryWrapperX<BpmTaskAssignRuleDO>()
                .eq("process_definition_id", processDefinitionId)
                .eqIfPresent("task_definition_key", taskDefinitionKey));
    }

    default BpmTaskAssignRuleDO selectListByModelIdAndTaskDefinitionKey(String modelId,
                                                                        String taskDefinitionKey) {
        return selectOne(new QueryWrapperX<BpmTaskAssignRuleDO>()
                .eq("model_id", modelId)
                .eq("process_definition_id", BpmTaskAssignRuleDO.PROCESS_DEFINITION_ID_NULL)
                .eq("task_definition_key", taskDefinitionKey));
    }

}