package cn.iocoder.yudao.module.bpm.service.meeting.room;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.metting.room.vo.OaMeetingRoomRespDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.meeting.room.vo.OaMeetingRoomPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.meeting.room.vo.OaMeetingRoomRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.meeting.room.vo.OaMeetingRoomSaveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.meeting.room.OaMeetingRoomDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.meeting.room.OaMeetingRoomMapper;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_MEETING_ROOM_NOT_EXISTS;

/**
 * 会议室信息管理 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaMeetingRoomServiceImpl implements OaMeetingRoomService {

    @Resource
    private OaMeetingRoomMapper oaMeetingRoomMapper;

    @Resource
    private AdminUserApi adminUserApi;

    @Override
    public Long createOaMeetingRoom(OaMeetingRoomSaveReqVO createReqVO) {
        // 插入
        OaMeetingRoomDO oaMeetingRoom = BeanUtils.toBean(createReqVO, OaMeetingRoomDO.class);
        oaMeetingRoom.setRoomUuid(IdUtil.fastSimpleUUID());
        // Long managerId = createReqVO.getManagerId();
        // if (managerId != null) {
        //     AdminUserRespDTO user = adminUserApi.getUser(managerId);
        //     if (user != null) {
        //         oaMeetingRoom.setManagerName(user.getNickname());
        //     }
        // }
        oaMeetingRoomMapper.insert(oaMeetingRoom);
        // 返回
        return oaMeetingRoom.getId();
    }

    @Override
    public void updateOaMeetingRoom(OaMeetingRoomSaveReqVO updateReqVO) {
        // 校验存在
        validateOaMeetingRoomExists(updateReqVO.getId());
        // 更新
        OaMeetingRoomDO updateObj = BeanUtils.toBean(updateReqVO, OaMeetingRoomDO.class);
        // Long managerId = updateObj.getManagerId();
        // if (managerId != null) {
        //     AdminUserRespDTO user = adminUserApi.getUser(managerId);
        //     if (user != null) {
        //         updateObj.setManagerName(user.getNickname());
        //     }
        // }
        oaMeetingRoomMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaMeetingRoom(Long id) {
        // 校验存在
        validateOaMeetingRoomExists(id);
        // 删除
        oaMeetingRoomMapper.deleteById(id);
    }

    private void validateOaMeetingRoomExists(Long id) {
        if (oaMeetingRoomMapper.selectById(id) == null) {
            throw exception(OA_MEETING_ROOM_NOT_EXISTS);
        }
    }

    @Override
    public OaMeetingRoomDO getOaMeetingRoom(Long id) {
        return oaMeetingRoomMapper.selectById(id);
    }

    @Override
    public PageResult<OaMeetingRoomDO> getOaMeetingRoomPage(OaMeetingRoomPageReqVO pageReqVO) {
        return oaMeetingRoomMapper.selectPage(pageReqVO);
    }

    @Override
    public List<OaMeetingRoomDO> getOaMeetingRoomList() {
        return oaMeetingRoomMapper.selectList();
    }

    @Override
    public List<OaMeetingRoomRespDTO> getOaMeetingRoomList(Collection<Long> ids) {
        if (CollUtil.isEmpty(ids)) {
            return Collections.emptyList();
        }
        List<OaMeetingRoomDO> oaMeetingRoomDOS = oaMeetingRoomMapper.selectBatchIds(ids);
        return BeanUtils.toBean(oaMeetingRoomDOS, OaMeetingRoomRespDTO.class);
    }

}