package cn.iocoder.yudao.module.bpm.service.oa.turnover;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.enums.InfoSourceEnum;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.turnover.vo.OaTurnoverPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.turnover.vo.OaTurnoverRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.turnover.vo.OaTurnoverSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.turnover.OaTurnoverDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.turnover.OaTurnOverMapper;
import cn.iocoder.yudao.module.bpm.enums.DictDataConstants;
import cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.bpm.enums.OaAuditStatusEnum;
import cn.iocoder.yudao.module.bpm.enums.ProcessKeyEnum;
import cn.iocoder.yudao.module.bpm.framework.flowable.core.enums.BpmConstants;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import cn.iocoder.yudao.module.bpm.util.OANotifyContentUtil;
import cn.iocoder.yudao.module.employee.api.EmployeeApi;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.relations.api.turnover.RelationsTurnoverApi;
import cn.iocoder.yudao.module.relations.api.turnover.dto.RelationsTurnoverCreateReqDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.PostApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.dept.dto.PostRespDTO;
import cn.iocoder.yudao.module.system.api.dicttenant.DictDataTenantApi;
import cn.iocoder.yudao.module.system.api.dicttenant.dto.DictDataTenantRespDTO;
import cn.iocoder.yudao.module.system.api.notify.NotifyMessageSendApi;
import cn.iocoder.yudao.module.system.api.permission.PermissionApi;
import cn.iocoder.yudao.module.system.api.permission.dto.DeptDataPermissionRespDTO;
import cn.iocoder.yudao.module.system.enums.dicttenant.DictTypeTenantEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import liquibase.pro.packaged.S;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_NOT_ALLOW_OPERATE_BY_OTHER;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_TURNOVER_NOT_EXISTS;

/**
 * 离职流程信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaTurnoverServiceImpl implements OaTurnoverService {


    /**
     * 对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = ProcessKeyEnum.OA_TURNOVER.getProcessKey();


    @Resource
    private OaTurnOverMapper oaTurnoverMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Resource
    private FileApi fileApi;

    @Resource
    private EmployeeApi employeeApi;

    @Resource
    private DeptApi deptApi;

    @Resource
    private PostApi postApi;

    @Resource
    private RelationsTurnoverApi relationsTurnoverApi;

    @Resource
    private NotifyMessageSendApi notifyMessageSendApi;

    @Resource
    private DictDataTenantApi dictDataTenantApi;

    @Resource
    private PermissionApi permissionApi;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long stagingOaTurnover(OaTurnoverSaveReqVO stagingReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 登录员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        // 离职人信息
        EmployeeRespDTO employee = employeeApi.getEmployeeById(stagingReqVO.getEmployeeId());
        if (loginEmployee == null || (stagingReqVO.getEmployeeId() != null && employee == null)) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        OaTurnoverDO oaTurnover = BeanUtils.toBean(stagingReqVO, OaTurnoverDO.class);
        if (oaTurnover.getId() != null) {
            OaTurnoverDO oaTurnoverDO = validateOaTurnoverExists(oaTurnover.getId());
            validateOperator(loginUserId, Long.valueOf(oaTurnoverDO.getCreator()));
        } else {
            // 创建单据uuid
            String uuid = IdUtil.fastSimpleUUID();
            oaTurnover.setTurnoverId(uuid);
        }
        if (employee != null) {
            oaTurnover.setEmployeeId(employee.getId());
            oaTurnover.setEmployeeName(employee.getName());
            oaTurnover.setEmployeePhone(employee.getPhone());
            oaTurnover.setDeptId(employee.getDeptId());
            oaTurnover.setPostId(employee.getPostId());
            oaTurnover.setPosition(employee.getPosition());
            LocalDate entryDate = employee.getEntryDate();
            if (entryDate != null) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                oaTurnover.setEntryDate(entryDate.format(formatter));
            }
        }
        oaTurnover.setUserId(loginUserId);
        oaTurnover.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_STAGING);
        oaTurnover.setInfoSource("0");
        oaTurnover.setApplyEmployeeId(loginEmployee.getId());
        oaTurnover.setApplyEmployeeName(loginEmployee.getName());
        // oaTurnover.setCreator(String.valueOf(loginEmployee.getId()));
        // 暂存不保存审批人信息
        oaTurnover.setStartUserSelectAssignees(null);
        // 连续暂存，回显时前端会直接显示上次缓存的数据，会让用户认为第一次暂存没保存审批人，第二次却保存了
        // 为了前后一致，暂存时如果选择了审批人，也保存一下
        List<Long> selectAssignees = stagingReqVO.getStartUserSelectAssignees();
        if (CollectionUtil.isNotEmpty(selectAssignees)) {
            String collect = selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(","));
            oaTurnover.setStartUserSelectAssignees(collect);
        } else {
            if (oaTurnover.getId() != null) {
                // 单独更新审批人字段
                oaTurnoverMapper.update(new LambdaUpdateWrapper<OaTurnoverDO>()
                        .eq(OaTurnoverDO::getId, oaTurnover.getId())
                        .set(OaTurnoverDO::getStartUserSelectAssignees, null));
            }
        }
        // 保存或更新表单信息
        if (oaTurnover.getId() == null) {
            oaTurnoverMapper.insert(oaTurnover);
        } else {
            validateOaTurnoverExists(oaTurnover.getId());
            oaTurnoverMapper.updateById(oaTurnover);
        }
        // 保存业务uuid到附件中
        fileApi.updateFileBiz(stagingReqVO.getFileIdList(), oaTurnover.getTurnoverId());

        return oaTurnover.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long commitOaTurnover(OaTurnoverSaveReqVO commitReqVO) {
        if (CollectionUtil.isEmpty(commitReqVO.getStartUserSelectAssignees())) {
            throw exception(ErrorCodeConstants.TASK_CREATE_FAIL_NO_START_SELECT_ASSIGNEE);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 登录员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        // 离职人信息
        EmployeeRespDTO employee = employeeApi.getEmployeeById(commitReqVO.getEmployeeId());
        if (loginEmployee == null || employee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }


        OaTurnoverDO oaTurnover = BeanUtils.toBean(commitReqVO, OaTurnoverDO.class);
        if (oaTurnover.getId() != null) {
            OaTurnoverDO oaTurnoverDO = validateOaTurnoverExists(oaTurnover.getId());
            validateOperator(loginUserId, Long.valueOf(oaTurnoverDO.getCreator()));
        } else {
            // 创建单据uuid
            String uuid = IdUtil.fastSimpleUUID();
            oaTurnover.setTurnoverId(uuid);
        }
        oaTurnover.setEmployeeId(employee.getId());
        oaTurnover.setEmployeeName(employee.getName());
        oaTurnover.setEmployeePhone(employee.getPhone());
        oaTurnover.setDeptId(employee.getDeptId());
        oaTurnover.setPostId(employee.getPostId());
        oaTurnover.setPosition(employee.getPosition());
        LocalDate entryDate = employee.getEntryDate();
        if (entryDate != null) {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            oaTurnover.setEntryDate(entryDate.format(formatter));
        }

        oaTurnover.setUserId(loginUserId);
        oaTurnover.setInfoSource("0");
        oaTurnover.setApplyEmployeeId(loginEmployee.getId());
        oaTurnover.setApplyEmployeeName(loginEmployee.getName());
        // oaTurnover.setCreator(String.valueOf(loginEmployee.getId()));
        // 保存或更新表单信息
        if (oaTurnover.getId() == null) {
            oaTurnoverMapper.insert(oaTurnover);
        } else {
            OaTurnoverDO oaTurnoverDO = validateOaTurnoverExists(oaTurnover.getId());
            if (StrUtil.isNotBlank(oaTurnoverDO.getProcInstId())) {
                throw exception(ErrorCodeConstants.PROCESS_INSTANCE_CREATE_FAIL_HAS_PROCESS);
            }
            oaTurnoverMapper.updateById(oaTurnover);
        }
        // 发起流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        processInstanceVariables.put("auditPass", "true");
        // 添加审批人信息
        Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
        List<Long> selectAssignees = commitReqVO.getStartUserSelectAssignees();
        startUserSelectAssignees.put("approver", selectAssignees);
        String processInstanceId = processInstanceApi
                .createProcessInstance(loginEmployee.getId(),
                        new BpmProcessInstanceCreateReqDTO()
                                .setProcessDefinitionKey(PROCESS_KEY)
                                .setVariables(processInstanceVariables)
                                .setBusinessKey(String.valueOf(oaTurnover.getId()))
                                .setStartUserSelectAssignees(startUserSelectAssignees));

        //获取下一个审批人
        Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

        // 审批同意
        BpmTaskApproveReqVO agreeReqVO = new BpmTaskApproveReqVO();
        agreeReqVO.setId(task.getId());
        agreeReqVO.setReason("[首次提交]");
        agreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_COMMITTED));
        taskService.setVariable(task.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_COMMITTED);
        taskService.setVariable(task.getId(), "auditPass", "true");
        bpmTaskService.approveTask(loginEmployee.getId(), agreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();
        if (nextTask == null) {
            throw exception(ErrorCodeConstants.TASK_CREATE_FAIL_NO_START_SELECT_ASSIGNEE);
        }
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        EmployeeRespDTO currentAuditEmployee = employeeApi.getEmployeeById(currentAuditEmployeeId);


        // 将工作流的流程实例ID、单据状态、最后一次审批时间、当前审批人更新到单据信息中
        oaTurnoverMapper.updateById(new OaTurnoverDO()
                .setId(oaTurnover.getId())
                .setProcInstId(processInstanceId)
                .setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_COMMITTED)
                .setFinalAuditDate(LocalDateTime.now())
                .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                .setCurrentAuditEmployeeName(currentAuditEmployee.getName())
                .setStartUserSelectAssignees(selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(","))));
        // 保存业务uuid到附件中
        fileApi.updateFileBiz(commitReqVO.getFileIdList(), oaTurnover.getTurnoverId());

        // 给审批人发送审批站内信
        Long assigneeId = -1L;
        String assigneeName = "";
        String applyName = loginEmployee.getName();
        String processName = ProcessKeyEnum.getByProcessKey(PROCESS_KEY);

        String assignee = nextTask.getAssignee();
        if (StrUtil.isNotBlank(assignee)) {
            EmployeeRespDTO employeeAssignee = employeeApi.getEmployeeById(Long.valueOf(assignee));
            if (employeeAssignee != null) {
                assigneeId = employeeAssignee.getId();
                assigneeName = employeeAssignee.getName();
            }
        }
        StringBuilder content = OANotifyContentUtil.buildContentSendToAssignee(assigneeName, applyName, processName);
        notifyMessageSendApi.sendSingleMessageToMemberForOA(assigneeId, content.toString());

        return oaTurnover.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long agreeOaTurnover(BpmTaskApproveReqVO agreeReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 根据登录人查询出对应的员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        if (loginEmployee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        Task currentTask = bpmTaskService.getTask(agreeReqVO.getId());
        if (currentTask == null) {
            throw exception(ErrorCodeConstants.TASK_NOT_EXISTS);
        }
        // 先更新为审批中
        taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITING);
        LambdaQueryWrapper<OaTurnoverDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(OaTurnoverDO::getProcInstId, currentTask.getProcessInstanceId());
        OaTurnoverDO oaTurnover = oaTurnoverMapper.selectOne(lambdaQueryWrapper);
        // 如果是最后一个人审批同意，设置流程审批状态为已审核
        String[] auditPersons = oaTurnover.getStartUserSelectAssignees().split(",");
        String lastAuditPerson = Arrays.stream(auditPersons)
                .reduce((first, second) -> second)
                .orElse(null);
        if (currentTask.getAssignee().equals(lastAuditPerson)) {
            // 更新为已审核(暂时未生效，未找到原因，先不处理)
            taskService.setVariable(agreeReqVO.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITED);
        }
        // 审批同意
        String reason = "[同意]";
        if (StrUtil.isNotBlank(agreeReqVO.getReason())) {
            reason = reason + agreeReqVO.getReason();
        }
        agreeReqVO.setReason(reason);
        agreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_AUDITING));
        taskService.setVariable(agreeReqVO.getId(), "auditPass", "true");
        bpmTaskService.approveTask(loginEmployee.getId(), agreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        LambdaUpdateWrapper<OaTurnoverDO> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        if (nextTask != null) {
            Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
            EmployeeRespDTO currentAuditEmployee = employeeApi.getEmployeeById(currentAuditEmployeeId);

            // 如果审批人重复，实际未结束，再次更新为审核中
            taskService.setVariable(nextTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITING);
            lambdaUpdateWrapper.eq(OaTurnoverDO::getId, oaTurnover.getId())
                    .set(OaTurnoverDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_AUDITING)
                    .set(OaTurnoverDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                    .set(OaTurnoverDO::getCurrentAuditEmployeeName, currentAuditEmployee.getName())
                    .set(OaTurnoverDO::getFinalAuditDate, LocalDateTime.now());

        } else {
            lambdaUpdateWrapper.eq(OaTurnoverDO::getId, oaTurnover.getId())
                    .set(OaTurnoverDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_AUDITED)
                    .set(OaTurnoverDO::getCurrentAuditEmployeeId, null)
                    .set(OaTurnoverDO::getCurrentAuditEmployeeName, null)
                    .set(OaTurnoverDO::getFinalAuditDate, LocalDateTime.now());
        }
        oaTurnoverMapper.update(lambdaUpdateWrapper);
        // 复制业务单据信息到业务模块单据表
        if(Objects.isNull(nextTask)) {
            // 查询申请单
            OaTurnoverDO turnover = oaTurnoverMapper.selectById(oaTurnover.getId());

            // 转换为DTO
            RelationsTurnoverCreateReqDTO turnoverCreateReqDTO = BeanUtils.toBean(turnover, RelationsTurnoverCreateReqDTO.class);
            turnoverCreateReqDTO.setStatus(0);
            turnoverCreateReqDTO.setCreatorEmployeeId(turnover.getApplyEmployeeId());
            turnoverCreateReqDTO.setCreatorEmployeeName(turnover.getApplyEmployeeName());
            turnoverCreateReqDTO.setInfoSource(InfoSourceEnum.FLOW.getSource());

            // 保存单据信息
            relationsTurnoverApi.createTurnoverInfoAfterAudit(turnoverCreateReqDTO);
        }

        // 发送站内信
        Long assignId = -1L;
        String assigneeName = "";
        Long applyId = -1L;
        String applyName = "";
        String processName = ProcessKeyEnum.getByProcessKey(PROCESS_KEY);

        EmployeeRespDTO applyEmployee = employeeApi.getEmployeeById(oaTurnover.getApplyEmployeeId());
        if (applyEmployee != null) {
            applyId = applyEmployee.getId();
            applyName = applyEmployee.getName();
        }
        if (nextTask != null) {
            EmployeeRespDTO employeeAssignee = employeeApi.getEmployeeById(Long.valueOf(nextTask.getAssignee()));
            if (employeeAssignee != null) {
                assignId = employeeAssignee.getId();
                assigneeName = employeeAssignee.getName();
            }
            // 给审批人发送审批站内信
            StringBuilder content = OANotifyContentUtil.buildContentSendToAssignee(assigneeName, applyName, processName);
            notifyMessageSendApi.sendSingleMessageToMemberForOA(assignId, content.toString());
        } else {
            // 给申请人发送审批完成站内信
            StringBuilder content = OANotifyContentUtil.buildContentSendToApplyForComplete(applyName, processName);
            notifyMessageSendApi.sendSingleMessageToMemberForOA(applyId, content.toString());
        }


        // 返回
        return 1L;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long disagreeOaTurnover(BpmTaskReturnReqVO disagreeReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 根据登录人查询出对应的员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        if (loginEmployee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        Task currentTask = bpmTaskService.getTask(disagreeReqVO.getId());

        disagreeReqVO.setTargetTaskDefinitionKey("modifyApply");
        disagreeReqVO.setReason("[驳回]" + disagreeReqVO.getReason());
        disagreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_RETURNED));
        taskService.setVariable(disagreeReqVO.getId(), "auditPass", "false");
        taskService.setVariable(disagreeReqVO.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_RETURNED);
        bpmTaskService.returnTask(loginEmployee.getId(), disagreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        EmployeeRespDTO currentAuditEmployee = employeeApi.getEmployeeById(currentAuditEmployeeId);

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaTurnoverDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaTurnoverDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                .set(OaTurnoverDO::getCurrentAuditEmployeeName, currentAuditEmployee.getName())
                .set(OaTurnoverDO::getFinalAuditDate, LocalDateTime.now())
                .set(OaTurnoverDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_RETURNED)
                .eq(OaTurnoverDO::getProcInstId, currentTask.getProcessInstanceId());
        oaTurnoverMapper.update(updateWrapper);

        // 给申请人发送驳回站内信
        String assigneeName = loginEmployee.getName();
        Long applyId = -1L;
        String applyName = "";
        String processName = ProcessKeyEnum.getByProcessKey(PROCESS_KEY);
        OaTurnoverDO oaTurnoverDO = oaTurnoverMapper.selectOne(OaTurnoverDO::getProcInstId, currentTask.getProcessInstanceId());
        if (oaTurnoverDO != null) {
            EmployeeRespDTO applyEmployee = employeeApi.getEmployeeById(oaTurnoverDO.getApplyEmployeeId());
            if (applyEmployee != null) {
                applyId = applyEmployee.getId();
                applyName = applyEmployee.getName();
            }
        }
        StringBuilder content = OANotifyContentUtil.buildContentSendToApplyForDisagree(assigneeName, applyName, processName);
        notifyMessageSendApi.sendSingleMessageToMemberForOA(applyId, content.toString());


        return 1L;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long revocationOaTurnover(BpmTaskApproveReqVO revocationReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 根据登录人查询出对应的员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        if (loginEmployee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        Task currentTask = bpmTaskService.getTask(revocationReqVO.getId());
        if (currentTask == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.TASK_NOT_EXISTS);
        }
        LambdaQueryWrapper<OaTurnoverDO> lambdaQueryWrapper = new LambdaQueryWrapper<OaTurnoverDO>()
                .eq(OaTurnoverDO::getProcInstId, currentTask.getProcessInstanceId());
        OaTurnoverDO oaTurnoverDO = oaTurnoverMapper.selectOne(lambdaQueryWrapper);
        if (!DictDataConstants.OA_AUDIT_STATUS_COMMITTED.equals(oaTurnoverDO.getAuditStatus())
                || !Objects.equals(String.valueOf(loginEmployee.getId()), oaTurnoverDO.getCreator())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.TASK_REVOCATION_NOT_ALLOWED);
        }

        BpmTaskReturnReqVO returnReqVO = BeanUtils.toBean(revocationReqVO, BpmTaskReturnReqVO.class);
        returnReqVO.setTargetTaskDefinitionKey("modifyApply");
        String reason = "[撤回]";
        if (StrUtil.isNotBlank(returnReqVO.getReason())) {
            reason = reason + returnReqVO.getReason();
        }
        returnReqVO.setReason(reason);
        returnReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_RECALLED));
        taskService.setVariable(currentTask.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_RECALLED);
        bpmTaskService.revocationTask(loginEmployee.getId(), returnReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        EmployeeRespDTO currentAuditEmployee = employeeApi.getEmployeeById(currentAuditEmployeeId);

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaTurnoverDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaTurnoverDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                .set(OaTurnoverDO::getCurrentAuditEmployeeName, currentAuditEmployee.getName())
                .set(OaTurnoverDO::getFinalAuditDate, LocalDateTime.now())
                .set(OaTurnoverDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_RECALLED)
                .eq(OaTurnoverDO::getProcInstId, currentTask.getProcessInstanceId());
        oaTurnoverMapper.update(updateWrapper);

        return 1L;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long reCommitOaTurnover(OaTurnoverSaveReqVO reCommitReqVO) {
        if (reCommitReqVO.getId() == null) {
            throw exception(ErrorCodeConstants.SERVICE_ID_NOT_EXISTS);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 登录员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        // 离职人信息
        EmployeeRespDTO employee = employeeApi.getEmployeeById(reCommitReqVO.getEmployeeId());
        if (loginEmployee == null || employee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        OaTurnoverDO oaTurnoverNew = BeanUtils.toBean(reCommitReqVO, OaTurnoverDO.class);
        OaTurnoverDO oaTurnoverOld = validateOaTurnoverExists(oaTurnoverNew.getId());
        oaTurnoverNew.setTurnoverId(oaTurnoverOld.getTurnoverId())
                .setProcInstId(oaTurnoverOld.getProcInstId());

        oaTurnoverNew.setEmployeeId(employee.getId());
        oaTurnoverNew.setEmployeeName(employee.getName());
        oaTurnoverNew.setEmployeePhone(employee.getPhone());
        oaTurnoverNew.setDeptId(employee.getDeptId());
        oaTurnoverNew.setPostId(employee.getPostId());
        oaTurnoverNew.setPosition(employee.getPosition());
        LocalDate entryDate = employee.getEntryDate();
        if (entryDate != null) {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            oaTurnoverNew.setEntryDate(entryDate.format(formatter));
        }

        oaTurnoverNew.setUserId(loginUserId);
        oaTurnoverNew.setApplyEmployeeId(loginEmployee.getId());
        oaTurnoverNew.setApplyEmployeeName(loginEmployee.getName());

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaTurnoverNew.getProcInstId()).singleResult();
        BpmTaskApproveReqVO approveReqVO = new BpmTaskApproveReqVO();
        approveReqVO.setId(currentTask.getId())
                .setReason("[再次提交]");
        approveReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_COMMITTED));
        if (CollectionUtil.isNotEmpty(reCommitReqVO.getStartUserSelectAssignees())) {
            // 添加审批人信息到流程参数中
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            List<Long> selectAssignees = reCommitReqVO.getStartUserSelectAssignees();
            startUserSelectAssignees.put("approver", selectAssignees);
            taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_START_USER_SELECT_ASSIGNEES, startUserSelectAssignees);
            // 保存审批人信息
            oaTurnoverNew.setStartUserSelectAssignees(selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(",")));
        }
        taskService.setVariable(currentTask.getId(), "auditPass", "true");
        taskService.setVariable(currentTask.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_COMMITTED);
        // 再次提交，和审批通过逻辑相同
        bpmTaskService.approveTask(loginEmployee.getId(), approveReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        EmployeeRespDTO currentAuditEmployee = employeeApi.getEmployeeById(currentAuditEmployeeId);
        oaTurnoverNew.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_COMMITTED)
                .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                .setCurrentAuditEmployeeName(currentAuditEmployee.getName())
                .setFinalAuditDate(LocalDateTime.now());
        oaTurnoverMapper.updateById(oaTurnoverNew);

        // 保存业务uuid到附件中
        fileApi.updateFileBiz(reCommitReqVO.getFileIdList(), oaTurnoverNew.getTurnoverId());

        // 给审批人发送审批站内信
        Long assigneeId = -1L;
        String assigneeName = "";
        String applyName = loginEmployee.getName();
        String processName = ProcessKeyEnum.getByProcessKey(PROCESS_KEY);

        String assignee = nextTask.getAssignee();
        if (StrUtil.isNotBlank(assignee)) {
            EmployeeRespDTO assigneeEmployee = employeeApi.getEmployeeById(Long.valueOf(assignee));
            if (assigneeEmployee != null) {
                assigneeId = assigneeEmployee.getId();
                assigneeName = assigneeEmployee.getName();
            }
        }
        StringBuilder content = OANotifyContentUtil.buildContentSendToAssignee(assigneeName, applyName, processName);
        notifyMessageSendApi.sendSingleMessageToMemberForOA(assigneeId, content.toString());


        return 1L;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long closeOaTurnover(Long id) {
        if (id == null) {
            throw exception(ErrorCodeConstants.SERVICE_ID_NOT_EXISTS);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 根据登录人查询出对应的员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        if (loginEmployee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        OaTurnoverDO oaTurnoverDO = validateOaTurnoverExists(id);
        if (!DictDataConstants.OA_AUDIT_STATUS_RETURNED.equals(oaTurnoverDO.getAuditStatus())
                && !DictDataConstants.OA_AUDIT_STATUS_RECALLED.equals(oaTurnoverDO.getAuditStatus())) {
            throw exception(ErrorCodeConstants.TASK_CLOSE_NOT_ALLOWED);
        }

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaTurnoverDO.getProcInstId()).singleResult();
        BpmTaskApproveReqVO approveReqVO = new BpmTaskApproveReqVO();
        approveReqVO.setId(currentTask.getId())
                .setReason("[关闭]");
        approveReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_CLOSED));
        taskService.setVariable(currentTask.getId(), "auditPass", "false");
        taskService.setVariable(currentTask.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_CLOSED);
        // 关闭，使用审批通过的方法实现
        bpmTaskService.approveTask(loginEmployee.getId(), approveReqVO);

        LambdaUpdateWrapper<OaTurnoverDO> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        lambdaUpdateWrapper.set(OaTurnoverDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_CLOSED)
                .set(OaTurnoverDO::getCurrentAuditEmployeeId, null)
                .set(OaTurnoverDO::getCurrentAuditEmployeeName, null)
                .set(OaTurnoverDO::getFinalAuditDate, LocalDateTime.now())
                .eq(OaTurnoverDO::getId, id);
        oaTurnoverMapper.update(lambdaUpdateWrapper);
        return 1L;
    }

    @Override
    public void deleteOaTurnover(Long id) {
        // 校验存在
        OaTurnoverDO oaTurnoverDO = validateOaTurnoverDelete(id);
        validateOperator(SecurityFrameworkUtils.getLoginUserId(), Long.valueOf(oaTurnoverDO.getCreator()));
        // 删除
        oaTurnoverMapper.deleteById(id);
    }

    @Override
    public OaTurnoverRespVO getOaTurnover(Long id) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // 根据登录人查询出对应的员工信息
        EmployeeRespDTO loginEmployee = employeeApi.getEmployeeByUserId(loginUserId);
        if (loginEmployee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        OaTurnoverDO oaTurnoverDO = oaTurnoverMapper.selectById(id);
        OaTurnoverRespVO oaTurnoverRespVO = BeanUtils.toBean(oaTurnoverDO, OaTurnoverRespVO.class);
        if (oaTurnoverDO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(oaTurnoverDO.getDeptId());
            if (dept != null) {
                oaTurnoverRespVO.setDeptName(dept.getName());
            }
        }
        // 职位
        if (oaTurnoverDO.getPostId() != null) {
            PostRespDTO post = postApi.getPost(oaTurnoverDO.getPostId());
            if (post != null) {
                oaTurnoverRespVO.setPosition(post.getName());
            }
        }
        // 状态
        oaTurnoverRespVO.setAuditStatusDesc(OaAuditStatusEnum.getByStatus(oaTurnoverDO.getAuditStatus()));
        // 离职原因
        if (StrUtil.isNotBlank(oaTurnoverDO.getTurnoverReason())) {
            List<DictDataTenantRespDTO> dictDataList = dictDataTenantApi.getDictDataList(DictTypeTenantEnum.TENANT_TURNOVER_REASON.getType());
            Map<String, String> dictDataMap = CollectionUtils.convertMap(dictDataList, DictDataTenantRespDTO::getValue, DictDataTenantRespDTO::getLabel);
            String turnoverReasonDesc = dictDataMap.get(oaTurnoverDO.getTurnoverReason());
            if (StrUtil.isNotBlank(turnoverReasonDesc)) {
                oaTurnoverRespVO.setTurnoverReasonDesc(turnoverReasonDesc);
            }
        }

        String procInstId = oaTurnoverDO.getProcInstId();
        if (StrUtil.isNotBlank(procInstId)) {
            Task task = taskService.createTaskQuery()
                    .processInstanceId(procInstId)
                    .taskAssignee(String.valueOf(loginEmployee.getId()))
                    .singleResult();
            if (DictDataConstants.OA_AUDIT_STATUS_COMMITTED.equals(oaTurnoverDO.getAuditStatus())) {
                // 如果是已提交，不限制任务处理人是当前登录人，用于直接撤回
                task = taskService.createTaskQuery()
                        .processInstanceId(procInstId)
                        .singleResult();
            }
            if (task != null) {
                oaTurnoverRespVO.setTaskId(task.getId());
            }
        }

        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(oaTurnoverDO.getTurnoverId());
        oaTurnoverRespVO.setFileList(fileList);
        List<Long> fileIdList = fileList.stream().map(FileDTO::getId).collect(Collectors.toList());
        oaTurnoverRespVO.setFileIdList(fileIdList);

        // 审批记录
        if (StrUtil.isNotBlank(oaTurnoverDO.getProcInstId())) {
            List<BpmTaskRespVO> auditRecordList = bpmTaskService.getAuditRecordListByProcessInstanceId(oaTurnoverDO.getProcInstId());
            oaTurnoverRespVO.setAuditRecordList(auditRecordList);
        }

        // 审批人
        String startUserSelectAssignees = oaTurnoverDO.getStartUserSelectAssignees();
        if (StrUtil.isNotEmpty(startUserSelectAssignees)) {
            List<Long> employeeIdList = Arrays.stream(startUserSelectAssignees.split(","))
                    .map(Long::valueOf).collect(Collectors.toList());
            List<EmployeeRespDTO> employeeRespDTOList = employeeApi.getEmployeeListByIds(employeeIdList);
            Map<Long, EmployeeRespDTO> longEmployeeRespDTOMap = CollectionUtils.convertMap(employeeRespDTOList, EmployeeRespDTO::getId);
            List<EmployeeRespDTO> auditUserList = new ArrayList<>();
            for (Long employeeId : employeeIdList) {
                EmployeeRespDTO employeeRespDTO = longEmployeeRespDTOMap.get(employeeId);
                if (employeeRespDTO != null) {
                    employeeRespDTO.setNickname(employeeRespDTO.getName());
                    auditUserList.add(employeeRespDTO);
                }
            }
            oaTurnoverRespVO.setAuditUserList(auditUserList);
        }

        return oaTurnoverRespVO;
    }

    @Override
    public OaTurnoverRespVO getOaTurnoverByProcInstId(String procInstId) {
        OaTurnoverDO oaTurnoverDO = oaTurnoverMapper.selectOne(OaTurnoverDO::getProcInstId, procInstId);
        OaTurnoverRespVO oaTurnoverRespVO = BeanUtils.toBean(oaTurnoverDO, OaTurnoverRespVO.class);
        if (oaTurnoverDO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(oaTurnoverDO.getDeptId());
            if (dept != null) {
                oaTurnoverRespVO.setDeptName(dept.getName());
            }
        }

        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(oaTurnoverDO.getTurnoverId());
        oaTurnoverRespVO.setFileList(fileList);
        List<Long> fileIdList = fileList.stream().map(FileDTO::getId).collect(Collectors.toList());
        oaTurnoverRespVO.setFileIdList(fileIdList);

        // 审批记录
        if (StrUtil.isNotBlank(oaTurnoverDO.getProcInstId())) {
            List<BpmTaskRespVO> auditRecordList = bpmTaskService.getAuditRecordListByProcessInstanceId(oaTurnoverDO.getProcInstId());
            oaTurnoverRespVO.setAuditRecordList(auditRecordList);
        }

        // 审批人
        String startUserSelectAssignees = oaTurnoverDO.getStartUserSelectAssignees();
        if (StrUtil.isNotEmpty(startUserSelectAssignees)) {
            List<Long> employeeIdList = Arrays.stream(startUserSelectAssignees.split(","))
                    .map(Long::valueOf).collect(Collectors.toList());
            List<EmployeeRespDTO> employeeRespDTOList = employeeApi.getEmployeeListByIds(employeeIdList);
            Map<Long, EmployeeRespDTO> longEmployeeRespDTOMap = CollectionUtils.convertMap(employeeRespDTOList, EmployeeRespDTO::getId);
            List<EmployeeRespDTO> auditUserList = new ArrayList<>();
            for (Long employeeId : employeeIdList) {
                EmployeeRespDTO employeeRespDTO = longEmployeeRespDTOMap.get(employeeId);
                if (employeeRespDTO != null) {
                    employeeRespDTO.setNickname(employeeRespDTO.getName());
                    auditUserList.add(employeeRespDTO);
                }
            }
            oaTurnoverRespVO.setAuditUserList(auditUserList);
        }

        return oaTurnoverRespVO;
    }

    @Override
    public PageResult<OaTurnoverRespVO> getOaTurnoverPage(OaTurnoverPageReqVO pageReqVO) {
        // 数据权限
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        DeptDataPermissionRespDTO deptDataPermission = permissionApi.getDeptDataPermission(loginUserId);
        pageReqVO.setDeptDataPermission(deptDataPermission);
        pageReqVO.setUserId(loginUserId);

        PageResult<OaTurnoverDO> oaTurnoverDOPageResult = oaTurnoverMapper.selectPage(pageReqVO);
        PageResult<OaTurnoverRespVO> oaTurnoverRespVOPageResult = BeanUtils.toBean(oaTurnoverDOPageResult, OaTurnoverRespVO.class);

        List<OaTurnoverRespVO> oaTurnoverRespVOList = oaTurnoverRespVOPageResult.getList();
        if (CollectionUtil.isNotEmpty(oaTurnoverRespVOList)) {
            List<Long> employeeIdList = oaTurnoverRespVOList.stream().map(OaTurnoverRespVO::getCurrentAuditEmployeeId).collect(Collectors.toList());
            List<EmployeeRespDTO> employeeRespDTOList = employeeApi.getEmployeeListByIds(employeeIdList);
            Map<Long, EmployeeRespDTO> longEmployeeRespDTOMap = CollectionUtils.convertMap(employeeRespDTOList, EmployeeRespDTO::getId);

            List<Long> deptIdList = oaTurnoverRespVOList.stream().map(OaTurnoverRespVO::getDeptId).collect(Collectors.toList());
            List<DeptRespDTO> deptList = deptApi.getDeptList(deptIdList);
            Map<Long, DeptRespDTO> deptMap = CollectionUtils.convertMap(deptList, DeptRespDTO::getId);

            // 职位
            List<Long> postIdList = oaTurnoverRespVOList.stream().map(OaTurnoverRespVO::getPostId).collect(Collectors.toList());
            List<PostRespDTO> postList = postApi.getPostList(postIdList);
            Map<Long, PostRespDTO> postMap = CollectionUtils.convertMap(postList, PostRespDTO::getId);

            // 状态
            Map<String, String> statusMap = Arrays.stream(OaAuditStatusEnum.values())
                    .collect(Collectors.toMap(OaAuditStatusEnum::getStatus, OaAuditStatusEnum::getStatusDesc));

            // 离职原因
            List<DictDataTenantRespDTO> dictDataList = dictDataTenantApi.getDictDataList(DictTypeTenantEnum.TENANT_TURNOVER_REASON.getType());
            Map<String, String> dictDataMap = CollectionUtils.convertMap(dictDataList, DictDataTenantRespDTO::getValue, DictDataTenantRespDTO::getLabel);


            for (OaTurnoverRespVO respVO : oaTurnoverRespVOList) {
                // 当前处理人
                if (respVO.getCurrentAuditEmployeeId() != null) {
                    EmployeeRespDTO employeeRespDTO = longEmployeeRespDTOMap.get(respVO.getCurrentAuditEmployeeId());
                    if (employeeRespDTO != null) {
                        respVO.setCurrentAuditEmployeeName(employeeRespDTO.getName());
                    }
                }
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO deptRespDTO = deptMap.get(respVO.getDeptId());
                    if (deptRespDTO != null) {
                        respVO.setDeptName(deptRespDTO.getName());
                    }
                }
                // 职位
                if (respVO.getPostId() != null) {
                    PostRespDTO postRespDTO = postMap.get(respVO.getPostId());
                    if (postRespDTO != null) {
                        respVO.setPosition(postRespDTO.getName());
                    }
                }
                // 状态
                respVO.setAuditStatusDesc(statusMap.get(respVO.getAuditStatus()));
                if (StrUtil.isNotBlank(respVO.getTurnoverReason())) {
                    String turnoverReasonDesc = dictDataMap.get(respVO.getTurnoverReason());
                    if (StrUtil.isNotBlank(turnoverReasonDesc)) {
                        respVO.setTurnoverReasonDesc(turnoverReasonDesc);
                    }
                }
            }
        }

        return oaTurnoverRespVOPageResult;
    }

    @Override
    public PageResult<OaTurnoverRespVO> exportOaTurnoverExcel(OaTurnoverPageReqVO pageReqVO) {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        return getOaTurnoverPage(pageReqVO);
    }

    private OaTurnoverDO validateOaTurnoverDelete(Long id) {
        OaTurnoverDO oaTurnoverDO = validateOaTurnoverExists(id);
        if (!DictDataConstants.OA_AUDIT_STATUS_STAGING.equals(oaTurnoverDO.getAuditStatus())) {
            throw exception(ErrorCodeConstants.DELETE_FAIL_NOT_STAGING);
        }
        return oaTurnoverDO;
    }

    private OaTurnoverDO validateOaTurnoverExists(Long id) {
        OaTurnoverDO oaTurnoverDO = oaTurnoverMapper.selectById(id);
        if (oaTurnoverDO == null) {
            throw exception(OA_TURNOVER_NOT_EXISTS);
        }
        return oaTurnoverDO;
    }

    private void validateOperator(Long loginUserId, Long creator) {
        if (!Objects.equals(loginUserId, creator)) {
            throw exception(OA_NOT_ALLOW_OPERATE_BY_OTHER);
        }
    }



}