package cn.iocoder.yudao.module.bpm.service.stamp.stampinfo;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo.vo.OaStampInfoPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo.vo.OaStampInfoRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.stamp.stampinfo.vo.OaStampInfoSaveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.stamp.stampinfo.OaStampInfoDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.stamp.stampinfo.OaStampInfoMapper;
import cn.iocoder.yudao.module.employee.api.EmployeeApi;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.system.api.dept.PostApi;
import cn.iocoder.yudao.module.system.api.permission.PermissionApi;
import cn.iocoder.yudao.module.system.api.permission.dto.DeptDataPermissionRespDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_LOGIN_USER_NOT_EXISTS;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_STAMP_INFO_NOT_EXISTS;
import static cn.iocoder.yudao.module.employee.enums.ErrorCodeConstants.EMPLOYEE_INFO_NOT_EXISTS;

/**
 * 用印信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaStampInfoServiceImpl implements OaStampInfoService {

    @Resource
    private OaStampInfoMapper oaStampInfoMapper;

    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private EmployeeApi employeeApi;

    @Resource
    private PostApi postApi;

    @Resource
    private FileApi fileApi;

    @Resource
    private PermissionApi permissionApi;

    @Override
    public Long createOaStampInfo(OaStampInfoSaveReqVO createReqVO) {
        // 登录人
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (loginUser == null) {
            throw exception(OA_LOGIN_USER_NOT_EXISTS);
        }
        // 插入准备
        OaStampInfoDO oaStampInfo = BeanUtils.toBean(createReqVO, OaStampInfoDO.class);
        // uuid
        oaStampInfo.setStampUuid(IdUtil.fastSimpleUUID());
        // 创建员工
        EmployeeRespDTO createEmployee = employeeApi.getEmployeeByUserId(loginUser.getId());
        if (Objects.isNull(createEmployee)) {
            throw exception(EMPLOYEE_INFO_NOT_EXISTS);
        }
        oaStampInfo.setCreateEmployeeId(createEmployee.getId());
        // 用印员工
        EmployeeRespDTO employee = employeeApi.getEmployeeById(createReqVO.getEmployeeId());
        if (Objects.isNull(employee)) {
            throw exception(EMPLOYEE_INFO_NOT_EXISTS);
        }
        // 部门
        oaStampInfo.setDeptId(employee.getDeptId());
        // 职位id
        oaStampInfo.setPostId(employee.getPostId());
        // 职位
        oaStampInfo.setPosition(employee.getPosition());
        // 印章
        List<Long> stampSealId = createReqVO.getStampSealId();
        if (CollUtil.isNotEmpty(stampSealId)) {
            String stampSealIdStr = stampSealId.stream().map(String::valueOf).collect(Collectors.joining(","));
            oaStampInfo.setStampSealId(stampSealIdStr);
        } else {
            oaStampInfo.setStampSealId(null);
        }
        // 状态
        oaStampInfo.setStatus("0"); // 已完成
        oaStampInfoMapper.insert(oaStampInfo);

        // 附件
        fileApi.updateFileBiz(createReqVO.getFileIdList(), oaStampInfo.getStampUuid());

        // 返回
        return oaStampInfo.getId();
    }

    @Override
    public Long createStampInfoAfterAudit(OaStampInfoDO stampInfoDO) {
        if (Objects.isNull(stampInfoDO)) {
            return -1L;
        }
        oaStampInfoMapper.insert(stampInfoDO);
        return stampInfoDO.getId();
    }

    @Override
    public void updateOaStampInfo(OaStampInfoSaveReqVO updateReqVO) {
        // 校验存在
        OaStampInfoDO stampInfoDOBefore = validateOaStampInfoExists(updateReqVO.getId());
        // 更新准备
        OaStampInfoDO updateObj = BeanUtils.toBean(updateReqVO, OaStampInfoDO.class);
        // 用印员工
        EmployeeRespDTO employee = employeeApi.getEmployeeById(updateReqVO.getEmployeeId());
        if (Objects.isNull(employee)) {
            throw exception(EMPLOYEE_INFO_NOT_EXISTS);
        }
        // 部门
        updateObj.setDeptId(employee.getDeptId());
        // 职位id
        updateObj.setPostId(employee.getPostId());
        // 职位
        updateObj.setPosition(employee.getPosition());
        // 印章
        List<Long> stampSealId = updateReqVO.getStampSealId();
        if (CollUtil.isNotEmpty(stampSealId)) {
            String stampSealIdStr = stampSealId.stream().map(String::valueOf).collect(Collectors.joining(","));
            updateObj.setStampSealId(stampSealIdStr);
        } else {
            updateObj.setStampSealId(null);
        }

        oaStampInfoMapper.updateById(updateObj);

        // 附件
        fileApi.updateFileBiz(updateReqVO.getFileIdList(), stampInfoDOBefore.getStampUuid());
    }

    @Override
    public void deleteOaStampInfo(Long id) {
        // 校验存在
        validateOaStampInfoExists(id);
        // 删除
        oaStampInfoMapper.deleteById(id);
    }

    @Override
    public void invalidOaStampInfo(Long id) {
        // 校验存在
        validateOaStampInfoExists(id);
        OaStampInfoDO oaStampInfoDO = OaStampInfoDO.builder()
                .status("1")
                .id(id)
                .build();
        oaStampInfoMapper.updateById(oaStampInfoDO);

    }

    private OaStampInfoDO validateOaStampInfoExists(Long id) {
        OaStampInfoDO oaStampInfoDO = oaStampInfoMapper.selectById(id);
        if (oaStampInfoDO == null) {
            throw exception(OA_STAMP_INFO_NOT_EXISTS);
        }
        return oaStampInfoDO;
    }

    @Override
    public OaStampInfoDO getOaStampInfo(Long id) {
        return oaStampInfoMapper.selectById(id);
    }

    @Override
    public PageResult<OaStampInfoRespVO> getOaStampInfoPage(OaStampInfoPageReqVO pageReqVO) {
        // 数据权限
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        DeptDataPermissionRespDTO deptDataPermission = permissionApi.getDeptDataPermission(loginUserId);
        pageReqVO.setDeptDataPermission(deptDataPermission);
        pageReqVO.setUserId(loginUserId);

        pageReqVO.setPageNo((pageReqVO.getPageNo() - 1) * pageReqVO.getPageSize());

        long pageCount = oaStampInfoMapper.selectPageCount(pageReqVO);
        List<OaStampInfoRespVO> pageList = oaStampInfoMapper.selectPageList(pageReqVO);

        for (OaStampInfoRespVO oaStampInfoRespVO : pageList) {
            // 状态描述
            String status = oaStampInfoRespVO.getStatus();
            switch (status) {
                case "0":
                    oaStampInfoRespVO.setStatusDesc("已完成");
                    break;
                case "1":
                    oaStampInfoRespVO.setStatusDesc("已作废");
                    break;
                default:
                    oaStampInfoRespVO.setStatusDesc("未知");
                    break;
            }
            // 数据来源描述
            String infoSource = oaStampInfoRespVO.getInfoSource();
            switch (infoSource) {
                case "0":
                    oaStampInfoRespVO.setInfoSourceDesc("流程添加");
                    break;
                case "1":
                    oaStampInfoRespVO.setInfoSourceDesc("手动添加");
                    break;
                default:
                    oaStampInfoRespVO.setInfoSourceDesc("未知");
                    break;
            }
        }
        return new PageResult<>(pageList, pageCount);
    }

}