CREATE TABLE IF NOT EXISTS "bpm_user_group" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "name" varchar(63) NOT NULL,
    "description" varchar(255) NOT NULL,
    "status" tinyint NOT NULL,
    "user_ids" varchar(255) NOT NULL,
    "creator" varchar(64) DEFAULT '',
    "create_time" timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar(64) DEFAULT '',
    "update_time" timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "deleted" bit NOT NULL DEFAULT FALSE,
    PRIMARY KEY ("id")
) COMMENT '用户组';

CREATE TABLE IF NOT EXISTS "bpm_category" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "name" varchar(63) NOT NULL,
    "code" varchar(63) NOT NULL,
    "description" varchar(255) NOT NULL,
    "status" tinyint NOT NULL,
    "sort" int NOT NULL,
    "creator" varchar(64) DEFAULT '',
    "create_time" timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar(64) DEFAULT '',
    "update_time" timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "deleted" bit NOT NULL DEFAULT FALSE,
    PRIMARY KEY ("id")
) COMMENT '分类';

CREATE TABLE IF NOT EXISTS "bpm_form" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "name" varchar(63) NOT NULL,
    "status" tinyint NOT NULL,
    "fields" varchar(255) NOT NULL,
    "conf" varchar(255) NOT NULL,
    "remark" varchar(255),
    "creator" varchar(64) DEFAULT '',
    "create_time" timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar(64) DEFAULT '',
    "update_time" timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "deleted" bit NOT NULL DEFAULT FALSE,
    PRIMARY KEY ("id")
) COMMENT '动态表单';

-- 将该建表 SQL 语句，添加到 yudao-module-bpm-biz 模块的 test/resources/sql/create_tables.sql 文件里
CREATE TABLE IF NOT EXISTS "bpm_oa_conversion" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "official_id" varchar,
    "employee_id" bigint,
    "employee_uuid" varchar,
    "employee_name" varchar,
    "user_id" bigint,
    "user_uuid" varchar,
    "dept_id" bigint,
    "dept_uuid" varchar,
    "position" varchar,
    "entry_date" varchar,
    "probation_end_date" varchar,
    "conversion_date" varchar,
    "conversion_reason" varchar,
    "work_summary" varchar,
    "remarks" varchar,
    "proc_inst_id" varchar,
    "audit_status" varchar,
    "current_audit_user_id" bigint,
    "current_audit_user_uuid" varchar,
    "current_audit_employee_id" bigint,
    "current_audit_employee_uuid" varchar,
    "final_audit_date" varchar,
    "info_source" varchar,
    "creator" varchar DEFAULT '',
    "create_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar DEFAULT '',
    "update_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    `deleted` int NOT NULL DEFAULT 0,
    "tenant_id" bigint NOT NULL default 0,
    PRIMARY KEY ("id")
    ) COMMENT '转正流程信息表';

-- 将该建表 SQL 语句，添加到 yudao-module-bpm-biz 模块的 test/resources/sql/create_tables.sql 文件里
CREATE TABLE IF NOT EXISTS "bpm_oa_renew" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "renew_id" varchar,
    "employee_id" bigint,
    "employee_uuid" varchar,
    "employee_name" varchar,
    "user_id" bigint,
    "user_uuid" varchar,
    "dept_id" bigint,
    "dept_uuid" varchar,
    "position" varchar,
    "old_contract_start_date" varchar,
    "old_contract_end_date" varchar,
    "renew_period" varchar,
    "renew_contract_start_date" varchar,
    "renew_contract_end_date" varchar,
    "renew_reason" varchar,
    "work_performance" varchar,
    "remarks" varchar,
    "proc_inst_id" varchar,
    "audit_status" varchar,
    "current_audit_user_id" bigint,
    "current_audit_user_uuid" varchar,
    "current_audit_employee_id" bigint,
    "current_audit_employee_uuid" varchar,
    "final_audit_date" varchar,
    "info_source" varchar,
    "creator" varchar DEFAULT '',
    "create_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar DEFAULT '',
    "update_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    `deleted` int NOT NULL DEFAULT 0,
    "tenant_id" bigint NOT NULL default 0,
    PRIMARY KEY ("id")
    ) COMMENT '合同续签流程信息表';
-- 将该建表 SQL 语句，添加到 yudao-module-bpm-biz 模块的 test/resources/sql/create_tables.sql 文件里
CREATE TABLE IF NOT EXISTS "bpm_oa_dimission" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "dimission_uuid" varchar,
    "employee_id" bigint,
    "employee_uuid" varchar,
    "employee_name" varchar,
    "user_id" bigint,
    "user_uuid" varchar,
    "dept_id" bigint,
    "dept_uuid" varchar,
    "position" varchar,
    "start_date" varchar,
    "dimission_date" varchar,
    "dimission_reason" varchar,
    "remarks" varchar,
    "proc_inst_id" varchar,
    "audit_status" varchar,
    "current_audit_user_id" bigint,
    "current_audit_user_uuid" varchar,
    "current_audit_employee_id" bigint,
    "current_audit_employee_uuid" varchar,
    "final_audit_date" varchar,
    "info_source" varchar,
    "creator" varchar DEFAULT '',
    "create_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar DEFAULT '',
    "update_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    `deleted` int NOT NULL DEFAULT 0,
    "tenant_id" bigint NOT NULL default 0,
    PRIMARY KEY ("id")
) COMMENT '离职流程信息表';
-- 将该建表 SQL 语句，添加到 yudao-module-bpm-biz 模块的 test/resources/sql/create_tables.sql 文件里
CREATE TABLE IF NOT EXISTS "bpm_oa_holiday" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "holiday_id" varchar,
    "employee_id" bigint,
    "employee_uuid" varchar,
    "employee_name" varchar,
    "user_id" bigint,
    "user_uuid" varchar,
    "dept_id" bigint,
    "dept_uuid" varchar,
    "position" varchar,
    "holiday_type" varchar,
    "start_date" varchar,
    "end_date" varchar,
    "day" varchar,
    "start_time" varchar,
    "end_time" varchar,
    "reason" varchar,
    "remarks" varchar,
    "proc_inst_id" varchar,
    "audit_status" varchar,
    "current_audit_user_id" bigint,
    "current_audit_user_uuid" varchar,
    "current_audit_employee_id" bigint,
    "current_audit_employee_uuid" varchar,
    "final_audit_date" varchar,
    "info_source" varchar,
    "creator" varchar DEFAULT '',
    "create_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar DEFAULT '',
    "update_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    "tenant_id" bigint NOT NULL,
    PRIMARY KEY ("id")
    ) COMMENT '请假流程信息表';
-- 将该建表 SQL 语句，添加到 yudao-module-bpm-biz 模块的 test/resources/sql/create_tables.sql 文件里
CREATE TABLE IF NOT EXISTS "bpm_oa_universal" (
    "id" bigint NOT NULL GENERATED BY DEFAULT AS IDENTITY,
    "universal_id" varchar,
    "employee_id" bigint,
    "employee_uuid" varchar,
    "employee_name" varchar,
    "user_id" bigint,
    "user_uuid" varchar,
    "dept_id" bigint,
    "dept_uuid" varchar,
    "position" varchar,
    "title" varchar,
    "description" varchar,
    "remarks" varchar,
    "proc_inst_id" varchar,
    "audit_status" varchar,
    "current_audit_user_id" bigint,
    "current_audit_user_uuid" varchar,
    "current_audit_employee_id" bigint,
    "current_audit_employee_uuid" varchar,
    "final_audit_date" varchar,
    "info_source" varchar,
    "creator" varchar DEFAULT '',
    "create_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "updater" varchar DEFAULT '',
    "update_time" datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    "deleted" bit NOT NULL DEFAULT FALSE,
    "tenant_id" bigint NOT NULL DEFAULT 0,
    PRIMARY KEY ("id")
    ) COMMENT '通用事项审批流程信息表';