package cn.iocoder.yudao.module.customer.controller.admin.businessopportunity;

import cn.iocoder.yudao.module.customer.controller.admin.info.vo.CustomerInfoRespVO;
import cn.iocoder.yudao.module.customer.service.info.CustomerInfoService;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.customer.controller.admin.businessopportunity.vo.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.businessopportunity.CustomerBusinessOpportunityDO;
import cn.iocoder.yudao.module.customer.service.businessopportunity.CustomerBusinessOpportunityService;

@Tag(name = "管理后台 - 客户商机")
@RestController
@RequestMapping("/customer/business-opportunity")
@Validated
public class CustomerBusinessOpportunityController {

    @Resource
    private CustomerBusinessOpportunityService businessOpportunityService;
    @Resource
    private DeptApi deptApi;
    @Resource
    private CustomerInfoService customerInfoService;

    @PostMapping("/create")
    @Operation(summary = "创建客户商机")
    @PreAuthorize("@ss.hasPermission('customer:business-opportunity:create')")
    public CommonResult<Long> createBusinessOpportunity(@Valid @RequestBody CustomerBusinessOpportunitySaveReqVO createReqVO) {
        return success(businessOpportunityService.createBusinessOpportunity(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新客户商机")
    @PreAuthorize("@ss.hasPermission('customer:business-opportunity:update')")
    public CommonResult<Boolean> updateBusinessOpportunity(@Valid @RequestBody CustomerBusinessOpportunitySaveReqVO updateReqVO) {
        businessOpportunityService.updateBusinessOpportunity(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除客户商机")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('customer:business-opportunity:delete')")
    public CommonResult<Boolean> deleteBusinessOpportunity(@RequestParam("id") Long id) {
        businessOpportunityService.deleteBusinessOpportunity(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得客户商机")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('customer:business-opportunity:query')")
    public CommonResult<CustomerBusinessOpportunityRespVO> getBusinessOpportunity(@RequestParam("id") Long id) {
        return success(businessOpportunityService.getById(id));
    }

    @GetMapping("/page")
    @Operation(summary = "获得客户商机分页")
    @PreAuthorize("@ss.hasPermission('customer:business-opportunity:query')")
    public CommonResult<PageResult<CustomerBusinessOpportunityRespVO>> getBusinessOpportunityPage(@Valid CustomerBusinessOpportunityPageReqVO pageReqVO) {
        PageResult<CustomerBusinessOpportunityDO> pageResult = businessOpportunityService.getBusinessOpportunityPage(pageReqVO);
        PageResult<CustomerBusinessOpportunityRespVO> result = BeanUtils.toBean(pageResult, CustomerBusinessOpportunityRespVO.class);
        if (result != null && result.getList() != null && result.getList().size() > 0) {
            result.getList().forEach(respVO -> {
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
                    if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                        respVO.setDeptName(dept.getName());
                    }
                }
                // 客户
                if (respVO.getCustomerId() != null) {
                    CustomerInfoRespVO infoRespVO = customerInfoService.getById(respVO.getCustomerId());
                    if (infoRespVO != null && StringUtils.isNotBlank(infoRespVO.getCustomerName())) {
                        respVO.setCustomerName(infoRespVO.getCustomerName());
                    }
                }
            });
        }
        return success(result);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出客户商机 Excel")
    @PreAuthorize("@ss.hasPermission('customer:business-opportunity:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportBusinessOpportunityExcel(@Valid CustomerBusinessOpportunityPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<CustomerBusinessOpportunityDO> list = businessOpportunityService.getBusinessOpportunityPage(pageReqVO).getList();
        List<CustomerBusinessOpportunityRespVO> result = BeanUtils.toBean(list, CustomerBusinessOpportunityRespVO.class);
        if (result != null && result.size() > 0) {
            result.forEach(respVO -> {
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
                    if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                        respVO.setDeptName(dept.getName());
                    }
                }
                // 客户
                if (respVO.getCustomerId() != null) {
                    CustomerInfoRespVO infoRespVO = customerInfoService.getById(respVO.getCustomerId());
                    if (infoRespVO != null && StringUtils.isNotBlank(infoRespVO.getCustomerName())) {
                        respVO.setCustomerName(infoRespVO.getCustomerName());
                    }
                }
            });
        }

        // 导出 Excel
        ExcelUtils.write(response, "客户商机.xls", "数据", CustomerBusinessOpportunityRespVO.class,
                result);
    }

    @GetMapping("/getOpportunityCode")
    @Operation(summary = "获取客户商机编号")
    @ApiOperationSupport(order = 1)
    public CommonResult<String> getOpportunityCode() {
        return success(businessOpportunityService.getOpportunityCode());
    }

}