package cn.iocoder.yudao.module.customer.service.info;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.controller.admin.info.vo.CustomerInfoPageReqVO;
import cn.iocoder.yudao.module.customer.controller.admin.info.vo.CustomerInfoRespVO;
import cn.iocoder.yudao.module.customer.controller.admin.info.vo.CustomerInfoSaveReqVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.info.CustomerInfoDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 客户信息 Service 接口
 *
 * @author zhaopq
 */
public interface CustomerInfoService {

    /**
     * 创建客户信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInfo(@Valid CustomerInfoSaveReqVO createReqVO);

    /**
     * 更新客户信息
     *
     * @param updateReqVO 更新信息
     */
    void updateInfo(@Valid CustomerInfoSaveReqVO updateReqVO);

    /**
     * 删除客户信息
     *
     * @param id 编号
     */
    void deleteInfo(Long id);

    /**
     * 获得客户信息
     *
     * @param id 编号
     * @return 客户信息
     */
    CustomerInfoDO getInfo(Long id);

    /**
     * 通过ID获得客户信息
     *
     * @param id
     * @return
     */
    CustomerInfoRespVO getById(Long id);

    /**
     * 获得客户信息分页
     *
     * @param pageReqVO 分页查询
     * @return 客户信息分页
     */
    PageResult<CustomerInfoDO> getInfoPage(CustomerInfoPageReqVO pageReqVO);

    /**
     * 获取客户编号
     * 客户编号是标识客户信息的唯一代码
     *
     * @return 客户编号
     */
    String getCustomerCode();

    /**
     * 修改开启状态
     *
     * @param id     用户编号
     * @param status 状态
     */
    void updateOpenStatus(Long id, Integer status);

    /**
     * 根据id集合获得客户信息列表,，包括开启的和关闭的
     *
     * @param ids 客户信息id集合
     * @return 客户信息列表
     */
    List<CustomerInfoDO> getCustomerInfoList(Collection<Long> ids);

    /**
     * 获得客户信息列表，用于其他模块
     *
     * @param isOpen 是否开启 0开启 1关闭
     * @return 客户信息列表
     */
    List<CustomerInfoDO> getCustomerInfoListForSelect(Integer isOpen);
}