package cn.iocoder.yudao.module.asset.enums;

/**
 * 资产信息单据状态枚举类
 */
public enum AssetStatusEnum {
    /**
     * 资产状态已完成
     */
    COMPLETED("0", "已完成"),
    /**
     * 资产状态已作废
     */
    ABANDONED("1", "已作废");

    // 状态码，用于系统中存储和传输资产状态的标识
    private final String status;
    // 状态描述，用于展示给用户看的状态文字
    private final String desc;

    /**
     * 枚举构造器，初始化状态码和状态描述
     *
     * @param status 状态码
     * @param desc   状态描述
     */
    AssetStatusEnum(String status, String desc) {
        this.status = status;
        this.desc = desc;
    }

    /**
     * 获取状态码
     *
     * @return 状态码
     */
    public String getStatus() {
        return status;
    }

    /**
     * 获取状态描述
     *
     * @return 状态描述
     */
    public String getDesc() {
        return desc;
    }
}
