package cn.iocoder.yudao.module.asset.service.assetinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.asset.controller.admin.assetinfo.vo.AssetInfoPageReqVO;
import cn.iocoder.yudao.module.asset.controller.admin.assetinfo.vo.AssetInfoRespVO;
import cn.iocoder.yudao.module.asset.controller.admin.assetinfo.vo.AssetInfoRetireReqVO;
import cn.iocoder.yudao.module.asset.controller.admin.assetinfo.vo.AssetInfoSaveReqVO;
import cn.iocoder.yudao.module.asset.dal.dataobject.assetinfo.AssetInfoDO;

import javax.validation.Valid;

/**
 * 固定资产信息 Service 接口
 *
 * @author dp
 */
public interface AssetInfoService {

    /**
     * 创建固定资产信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInfo(@Valid AssetInfoSaveReqVO createReqVO);

    /**
     * 更新固定资产信息
     *
     * @param updateReqVO 更新信息
     */
    void updateInfo(@Valid AssetInfoSaveReqVO updateReqVO);

    /**
     * 更新固定资产信息
     *
     * @param updateReqVO 更新信息
     */
    void availableOrOccupiedAssetInfo(AssetInfoDO updateReqVO);

    /**
     * 删除固定资产信息
     *
     * @param id 编号
     */
    void deleteInfo(Long id);

    /**
     * 获得固定资产信息
     *
     * @param id 编号
     * @return 固定资产信息
     */
    AssetInfoDO getInfo(Long id);

    /**
     * 获得固定资产信息分页
     *
     * @param pageReqVO 分页查询
     * @return 固定资产信息分页
     */
    PageResult<AssetInfoRespVO> getInfoPage(AssetInfoPageReqVO pageReqVO);

    /**
     * 废弃固定资产信息
     *
     * @param retireReqVO 报废信息
     */
    void retireInfo(AssetInfoRetireReqVO retireReqVO);
}