package cn.iocoder.yudao.module.cash.controller.admin.paymentinfo;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.cash.controller.admin.paymentinfo.vo.PaymentInfoPageReqVO;
import cn.iocoder.yudao.module.cash.controller.admin.paymentinfo.vo.PaymentInfoRespVO;
import cn.iocoder.yudao.module.cash.controller.admin.paymentinfo.vo.PaymentInfoSaveReqVO;
import cn.iocoder.yudao.module.cash.dal.dataobject.paymentinfo.PaymentInfoDO;
import cn.iocoder.yudao.module.cash.service.paymentinfo.PaymentInfoService;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 付款信息")
@RestController
@RequestMapping("/finance/cash/payment-info")
@Validated
public class PaymentInfoController {

    @Resource
    private PaymentInfoService paymentInfoService;

    @Resource
    private FileApi fileApi;

    @PostMapping("/create")
    @Operation(summary = "创建付款信息")
    @ApiOperationSupport(order = 1)
    // @PreAuthorize("@ss.hasPermission('cash:payment-info:create')")
    public CommonResult<Long> createPaymentInfo(@Valid @RequestBody PaymentInfoSaveReqVO createReqVO) {
        return success(paymentInfoService.createPaymentInfo(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新付款信息")
    @ApiOperationSupport(order = 2)
    // @PreAuthorize("@ss.hasPermission('cash:payment-info:update')")
    public CommonResult<Boolean> updatePaymentInfo(@Valid @RequestBody PaymentInfoSaveReqVO updateReqVO) {
        paymentInfoService.updatePaymentInfo(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除付款信息")
    @ApiOperationSupport(order = 3)
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('cash:payment-info:delete')")
    public CommonResult<Boolean> deletePaymentInfo(@RequestParam("id") Long id) {
        paymentInfoService.deletePaymentInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得付款信息")
    @ApiOperationSupport(order = 4)
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    // @PreAuthorize("@ss.hasPermission('cash:payment-info:query')")
    public CommonResult<PaymentInfoRespVO> getPaymentInfo(@RequestParam("id") Long id) {
        PaymentInfoDO paymentInfo = paymentInfoService.getPaymentInfo(id);
        PaymentInfoRespVO paymentInfoRespVO = BeanUtils.toBean(paymentInfo, PaymentInfoRespVO.class);

        String status = paymentInfoRespVO.getStatus();
        if ("0".equals(status)) {
            paymentInfoRespVO.setStatusDesc("已完成");
        } else if ("1".equals(status)) {
            paymentInfoRespVO.setStatusDesc("作废");
        } else {
            paymentInfoRespVO.setStatusDesc("未知");
        }

        // 附件
        List<FileDTO> fileDTOList = fileApi.getFileDTOListByBiz(paymentInfo.getPaymentInfoUuid());
        paymentInfoRespVO.setFileList(fileDTOList);

        return success(paymentInfoRespVO);
    }

    @GetMapping("/page")
    @Operation(summary = "获得付款信息分页")
    @ApiOperationSupport(order = 5)
    // @PreAuthorize("@ss.hasPermission('cash:payment-info:query')")
    public CommonResult<PageResult<PaymentInfoRespVO>> getPaymentInfoPage(@Valid @ParameterObject PaymentInfoPageReqVO pageReqVO) {
        PageResult<PaymentInfoRespVO> pageResult = paymentInfoService.getPaymentInfoPage(pageReqVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出付款信息 Excel")
    @ApiOperationSupport(order = 6)
    // @PreAuthorize("@ss.hasPermission('cash:payment-info:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportPaymentInfoExcel(@Valid @ParameterObject PaymentInfoPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<PaymentInfoRespVO> list = paymentInfoService.getPaymentInfoPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "付款信息.xls", "数据", PaymentInfoRespVO.class,
                        list);
    }

}