package cn.iocoder.yudao.module.expense.controller.admin.expenseinfo;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.employee.api.EmployeeApi;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;
import cn.iocoder.yudao.module.expense.controller.admin.expenseinfo.vo.ExpenseInfoPageReqVO;
import cn.iocoder.yudao.module.expense.controller.admin.expenseinfo.vo.ExpenseInfoRespVO;
import cn.iocoder.yudao.module.expense.controller.admin.expenseinfo.vo.ExpenseInfoSaveReqVO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expenseinfo.ExpenseInfoDO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expenseinfo.ExpenseInfoObjDO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expenseitem.ExpenseItemDO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expensetype.ExpenseTypeDO;
import cn.iocoder.yudao.module.expense.service.expenseinfo.ExpenseInfoService;
import cn.iocoder.yudao.module.expense.service.expenseitem.ExpenseItemService;
import cn.iocoder.yudao.module.expense.service.expensetype.ExpenseTypeService;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.PostApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.dept.dto.PostRespDTO;
import cn.iocoder.yudao.module.system.api.permission.PermissionApi;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 报销信息")
@RestController
@RequestMapping("/finance/expense/info")
@Validated
public class ExpenseInfoController {

    @Resource
    private ExpenseInfoService infoService;

    @Resource
    private ExpenseTypeService expenseTypeService;

    @Resource
    private ExpenseItemService expenseItemService;

    @Resource
    private DeptApi deptApi;

    @Resource
    private PostApi postApi;

    @Resource
    private EmployeeApi employeeApi;

    @Resource
    private FileApi fileApi;

    @PostMapping("/create")
    @Operation(summary = "创建报销信息")
    // @PreAuthorize("@ss.hasPermission('expense:info:create')")
    public CommonResult<Long> createInfo(@Valid @RequestBody ExpenseInfoSaveReqVO createReqVO) {
        createReqVO.setInfoSource("1");
        return success(infoService.createInfo(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新报销信息")
    // @PreAuthorize("@ss.hasPermission('expense:info:update')")
    public CommonResult<Boolean> updateInfo(@Valid @RequestBody ExpenseInfoSaveReqVO updateReqVO) {
        updateReqVO.setInfoSource("1");
        infoService.updateInfo(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除报销信息")
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('expense:info:delete')")
    public CommonResult<Boolean> deleteInfo(@RequestParam("id") Long id) {
        infoService.deleteInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得报销信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1")
    // @PreAuthorize("@ss.hasPermission('expense:info:query')")
    public CommonResult<ExpenseInfoRespVO> getInfo(@RequestParam("id") Long id) {
        // 从报销服务中获取报销信息
        ExpenseInfoDO info = infoService.getInfo(id);
        // 将报销信息转换为响应视图对象
        ExpenseInfoRespVO expenseInfoRespVO = BeanUtils.toBean(info, ExpenseInfoRespVO.class);

        // 获取并设置员工信息
        Long employeeId = info.getEmployeeId();
        EmployeeRespDTO employee = employeeApi.getEmployeeById(employeeId);
        if (Objects.nonNull(employee)) {
            expenseInfoRespVO.setEmployeeName(employee.getName());
            expenseInfoRespVO.setEmployeePhone(employee.getPhone());
        }

        // 获取并设置部门信息
        Long deptId = info.getDeptId();
        DeptRespDTO dept = deptApi.getDept(deptId);
        if (Objects.nonNull(dept)) {
            expenseInfoRespVO.setDeptName(dept.getName());
        }

        // 获取并设置职位信息
        Long postId = info.getPostId();
        PostRespDTO post = postApi.getPost(postId);
        if (Objects.nonNull(post)) {
            expenseInfoRespVO.setPosition(post.getName());
        }

        // 获取并设置报销类型信息
        Long expenseTypeId = info.getExpenseTypeId();
        ExpenseTypeDO expenseTypeDO = expenseTypeService.getType(expenseTypeId);
        if (expenseTypeDO != null) {
            expenseInfoRespVO.setExpenseTypeName(expenseTypeDO.getName());
        }

        // 根据状态码设置状态描述
        String status = info.getStatus();
        if (Objects.equals(status, "0")) {
            expenseInfoRespVO.setStatusDesc("已完成");
        } else if (Objects.equals(status, "1")) {
            expenseInfoRespVO.setStatusDesc("已作废");
        } else {
            expenseInfoRespVO.setStatusDesc("未知");
        }

        // 根据信息来源码设置信息来源描述
        String infoSource = info.getInfoSource();
        if (Objects.equals(infoSource, "0")) {
            expenseInfoRespVO.setInfoSourceDesc("流程添加");
        } else if (Objects.equals(infoSource, "1")) {
            expenseInfoRespVO.setInfoSourceDesc("手动添加");
        } else {
            expenseInfoRespVO.setInfoSourceDesc("未知来源");
        }

        // 获取并设置创建员工信息
        Long createEmployeeId = info.getCreateEmployeeId();
        EmployeeRespDTO createEmployee = employeeApi.getEmployeeById(createEmployeeId);
        if (Objects.nonNull(createEmployee)) {
            expenseInfoRespVO.setCreateEmployeeName(createEmployee.getName());
        }

        // 获取报销明细信息并补充报销项目名称
        List<ExpenseInfoObjDO> expenseInfoObjDOs = infoService.getExpenseInfoObjListByExpenseId(info.getId());
        List<Long> expenseItemIdList = expenseInfoObjDOs.stream().map(ExpenseInfoObjDO::getExpenseItemId).collect(Collectors.toList());
        List<ExpenseItemDO> expenseItemDOList = expenseItemService.getExpenseItemList(expenseItemIdList);
        Map<Long, ExpenseItemDO> longExpenseItemDOMap = CollectionUtils.convertMap(expenseItemDOList, ExpenseItemDO::getId);
        for (ExpenseInfoObjDO expenseInfoObjDO : expenseInfoObjDOs) {
            ExpenseItemDO expenseItemDO = longExpenseItemDOMap.get(expenseInfoObjDO.getExpenseItemId());
            if (Objects.nonNull(expenseItemDO)) {
                expenseInfoObjDO.setExpenseItemName(expenseItemDO.getName());
            }
        }
        expenseInfoRespVO.setExpenseInfoObjs(expenseInfoObjDOs);

        // 附件
        List<FileDTO> fileDTOListByBiz = fileApi.getFileDTOListByBiz(info.getExpenseUuid());
        expenseInfoRespVO.setFileList(fileDTOListByBiz);
        List<Long> fileIdList = fileDTOListByBiz.stream().map(FileDTO::getId).collect(Collectors.toList());
        expenseInfoRespVO.setFileIdList(fileIdList);

        // 返回成功响应
        return success(expenseInfoRespVO);
    }

    @GetMapping("/page")
    @Operation(summary = "获得报销信息分页")
    // @PreAuthorize("@ss.hasPermission('expense:info:query')")
    public CommonResult<PageResult<ExpenseInfoRespVO>> getInfoPage(@Valid @ParameterObject ExpenseInfoPageReqVO pageReqVO) {
        PageResult<ExpenseInfoRespVO> infoPage = infoService.getInfoPage(pageReqVO);
        return success(infoPage);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出报销信息 Excel")
    // @PreAuthorize("@ss.hasPermission('expense:info:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportInfoExcel(@Valid ExpenseInfoPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<ExpenseInfoRespVO> list = infoService.getInfoPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "报销信息.xls", "数据", ExpenseInfoRespVO.class,
                        list);
    }

    // ==================== 子表（报销信息子） ====================

    @GetMapping("/expense-info-obj/list-by-expense-id")
    @Operation(summary = "获得报销信息子列表")
    @Parameter(name = "expenseId", description = "报销主表主键id")
    // @PreAuthorize("@ss.hasPermission('expense:info:query')")
    public CommonResult<List<ExpenseInfoObjDO>> getExpenseInfoObjListByExpenseId(@RequestParam("expenseId") Long expenseId) {
        return success(infoService.getExpenseInfoObjListByExpenseId(expenseId));
    }

}