package cn.iocoder.yudao.module.expense.service.expensetype;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.expense.controller.admin.expensetype.vo.ExpenseTypePageReqVO;
import cn.iocoder.yudao.module.expense.controller.admin.expensetype.vo.ExpenseTypeSaveReqVO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expenseitem.ExpenseItemDO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expensetype.ExpenseTypeDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 报销类型信息 Service 接口
 *
 * @author dp
 */
public interface ExpenseTypeService {

    /**
     * 创建报销类型信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createType(@Valid ExpenseTypeSaveReqVO createReqVO);

    /**
     * 更新报销类型信息
     *
     * @param updateReqVO 更新信息
     */
    void updateType(@Valid ExpenseTypeSaveReqVO updateReqVO);

    /**
     * 删除报销类型信息
     *
     * @param id 编号
     */
    void deleteType(Long id);

    /**
     * 获得报销类型信息
     *
     * @param id 编号
     * @return 报销类型信息
     */
    ExpenseTypeDO getType(Long id);

    /**
     * 获得报销类型信息分页
     *
     * @param pageReqVO 分页查询
     * @return 报销类型信息分页
     */
    PageResult<ExpenseTypeDO> getTypePage(ExpenseTypePageReqVO pageReqVO);

    List<ExpenseTypeDO> getTypeList();

    List<ExpenseTypeDO> getExpenseTypeList(Collection<Long> ids);

    // ==================== 子表（报销费用项目信息） ====================

    /**
     * 获得报销费用项目信息列表
     *
     * @param expenseTypeId 报销类型主键id
     * @return 报销费用项目信息列表
     */
    List<ExpenseItemDO> getItemListByExpenseTypeId(Long expenseTypeId);

}