package cn.iocoder.yudao.module.attendance.controller.admin.info.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import lombok.*;

import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import io.swagger.v3.oas.annotations.media.Schema;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import java.math.BigDecimal;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import java.time.LocalDateTime;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Schema(description = "管理后台 - 考勤信息分页 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class AttendanceInfoPageReqVO extends PageParam {

    @Schema(description = "员工ID", example = "6960")
    private Long employeeId;

    @Schema(description = "员工姓名", requiredMode = Schema.RequiredMode.REQUIRED, example = "王五")
    private String employeeName;

    @Schema(description = "员工手机号", requiredMode = Schema.RequiredMode.REQUIRED, example = "13812345678")
    private String employeeMobile;

    @Schema(description = "部门ID", example = "13463")
    private Long deptId;

    @Schema(description = "部门名称", example = "13463")
    private String deptName;

    @Schema(description = "职位ID", example = "13463")
    private Long postId;

    @Schema(description = "职位名称", example = "13463")
    private String position;

    @Schema(description = "考勤日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] attendanceDate;

    @Schema(description = "上班时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalTime[] workStartTime;

    @Schema(description = "下班时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalTime[] workEndTime;

    @Schema(description = "迟到时间（如有）")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalTime[] lateTime;

    @Schema(description = "早退时间（如有）")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalTime[] leaveEarlyTime;

    @Schema(description = "加班时长（小时）")
    private BigDecimal overtimeDuration;

    @Schema(description = "请假类型", example = "2")
    private String leaveType;

    @Schema(description = "请假时长（小时或天数）")
    private BigDecimal leaveDuration;

    @Schema(description = "考勤状态", example = "1")
    private String attendanceStatus;

    @Schema(description = "备注")
    private String remarks;

    @Schema(description = "状态", example = "1")
    private Integer status;

    @Schema(description = "创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] createTime;

    @Schema(description = "租户编号", example = "1")
    private Long tenantId;

}