package cn.iocoder.yudao.module.attendance.controller.admin.info.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import java.util.*;
import java.math.BigDecimal;
import org.springframework.format.annotation.DateTimeFormat;
import java.time.LocalDateTime;
import com.alibaba.excel.annotation.*;

import javax.validation.constraints.NotEmpty;

@Schema(description = "管理后台 - 考勤信息 Response VO")
@Data
@ExcelIgnoreUnannotated
public class AttendanceInfoRespVO {

    @Schema(description = "考勤ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "14247")
    private Long id;

    @Schema(description = "员工ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "6960")
    private Long employeeId;

    @Schema(description = "员工姓名", requiredMode = Schema.RequiredMode.REQUIRED, example = "王五")
    @ExcelProperty("员工姓名")
    private String employeeName;

    @Schema(description = "部门名称", example = "13463")
    @ExcelProperty("部门")
    private String deptName;

    @Schema(description = "职位名称", example = "13463")
    @ExcelProperty("职位")
    private String position;

    @Schema(description = "员工手机号", requiredMode = Schema.RequiredMode.REQUIRED, example = "13812345678")
    @ExcelProperty("手机号")
    private String employeeMobile;

    @Schema(description = "考勤日期", requiredMode = Schema.RequiredMode.REQUIRED)
    @ExcelProperty("日期")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate attendanceDate;

    @Schema(description = "上班时间")
    @ExcelProperty("上班打卡")
    @JsonFormat(pattern = "HH:mm")
    private LocalTime workStartTime;

    @Schema(description = "下班时间")
    @ExcelProperty("下班打卡")
    @JsonFormat(pattern = "HH:mm")
    private LocalTime workEndTime;

    @Schema(description = "迟到时间（如有）")
    @JsonFormat(pattern = "HH:mm")
    private LocalTime lateTime;

    @Schema(description = "早退时间（如有）")
    @JsonFormat(pattern = "HH:mm")
    private LocalTime leaveEarlyTime;

    @Schema(description = "加班时长（小时）")
    private BigDecimal overtimeDuration;

    @Schema(description = "请假类型", example = "2")
    private String leaveType;

    @Schema(description = "请假时长（小时或天数）")
    private BigDecimal leaveDuration;

    @Schema(description = "考勤状态", example = "1")
    private String attendanceStatus;

    @Schema(description = "备注")
    private String remarks;

    @Schema(description = "状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
    private Integer status;

    @Schema(description = "创建时间", requiredMode = Schema.RequiredMode.REQUIRED)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    @Schema(description = "租户编号", example = "1")
    private Long tenantId;

}