package cn.iocoder.yudao.module.attendance.controller.admin.info.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

@Schema(description = "管理后台 - 考勤信息新增/修改 Request VO")
@Data
public class AttendanceInfoSaveReqVO {

    @Schema(description = "考勤ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "14247")
    private Long id;

    @Schema(description = "员工ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "6960")
    @NotNull(message = "员工ID不能为空")
    private Long employeeId;

    @Schema(description = "员工姓名", requiredMode = Schema.RequiredMode.REQUIRED, example = "王五")
    @NotEmpty(message = "员工姓名不能为空")
    private String employeeName;

    @Schema(description = "员工手机号", requiredMode = Schema.RequiredMode.REQUIRED, example = "13812345678")
    @NotEmpty(message = "员工手机号不能为空")
    private String employeeMobile;

    @Schema(description = "部门ID", example = "13463")
    private Long deptId;

    @Schema(description = "部门名称", example = "13463")
    private String deptName;

    @Schema(description = "职位ID", example = "13463")
    private Long postId;

    @Schema(description = "职位名称", example = "13463")
    private String position;

    @Schema(description = "考勤日期", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "考勤日期不能为空")
    private LocalDate attendanceDate;

    @Schema(description = "考勤月份", example = "2024-08")
    private String attendanceMonth;

    @Schema(description = "上班时间")
    private LocalTime workStartTime;

    @Schema(description = "下班时间")
    private LocalTime workEndTime;

    @Schema(description = "迟到时间（如有）")
    private LocalTime lateTime;

    @Schema(description = "早退时间（如有）")
    private LocalTime leaveEarlyTime;

    @Schema(description = "加班时长（小时）")
    private BigDecimal overtimeDuration;

    @Schema(description = "请假类型", example = "2")
    private String leaveType;

    @Schema(description = "请假时长（小时或天数）")
    private BigDecimal leaveDuration;

    @Schema(description = "考勤状态", example = "1")
    private String attendanceStatus;

    @Schema(description = "备注")
    private String remarks;

    @Schema(description = "状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
    private Integer status;

    @Schema(description = "租户编号", example = "1")
    private Long tenantId;

}