package cn.iocoder.yudao.module.attendance.service.employeesetting;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.attendance.controller.admin.employeesetting.vo.AttendanceEmployeeSettingBatchSaveReqVO;
import cn.iocoder.yudao.module.attendance.controller.admin.employeesetting.vo.AttendanceEmployeeSettingPageReqVO;
import cn.iocoder.yudao.module.attendance.controller.admin.employeesetting.vo.AttendanceEmployeeSettingRespVO;
import cn.iocoder.yudao.module.attendance.controller.admin.employeesetting.vo.AttendanceEmployeeSettingSaveReqVO;
import cn.iocoder.yudao.module.attendance.dal.dataobject.employeesetting.AttendanceEmployeeSettingDO;

import javax.validation.Valid;
import java.util.*;

/**
 * 考勤员工设置 Service 接口
 *
 * @author zhaopq
 */
public interface AttendanceEmployeeSettingService {

    /**
     * 创建考勤员工设置
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createEmployeeSetting(@Valid AttendanceEmployeeSettingSaveReqVO createReqVO);

    /**
     * 更新考勤员工设置
     *
     * @param updateReqVO 更新信息
     */
    void updateEmployeeSetting(@Valid AttendanceEmployeeSettingSaveReqVO updateReqVO);

    /**
     * 删除考勤员工设置
     *
     * @param id 编号
     */
    void deleteEmployeeSetting(Long id);

    /**
     * 获得考勤员工设置
     *
     * @param id 编号
     * @return 考勤员工设置
     */
    AttendanceEmployeeSettingDO getEmployeeSetting(Long id);

    /**
     * 获得考勤员工设置分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤员工设置分页
     */
    PageResult<AttendanceEmployeeSettingDO> getEmployeeSettingPage(AttendanceEmployeeSettingPageReqVO pageReqVO);

    /**
     * 获得考勤员工设置分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤员工设置分页
     */
    PageResult<AttendanceEmployeeSettingRespVO> getEmployeeInfoSettingPage(AttendanceEmployeeSettingPageReqVO pageReqVO);
    /**
     * 通过员工ID获得考勤员工设置
     *
     * @param employeeId 员工ID
     * @return 考勤员工设置
     */
    AttendanceEmployeeSettingRespVO getByEmployeeId(Long employeeId);

    /**
     * 更新考勤员工设置
     *
     * @param updateReqVO 更新信息
     */
    void batchUpdateEmployeeSetting(@Valid AttendanceEmployeeSettingBatchSaveReqVO updateReqVO);


}