package cn.iocoder.yudao.module.attendance.service.info;

import java.time.LocalDate;
import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.module.attendance.controller.admin.info.vo.*;
import cn.iocoder.yudao.module.attendance.dal.dataobject.business.AttendanceBusinessDO;
import cn.iocoder.yudao.module.attendance.dal.dataobject.info.AttendanceInfoDO;

import javax.validation.Valid;

/**
 * 考勤信息 Service 接口
 *
 * @author zhaopq
 */
public interface AttendanceInfoService {

    /**
     * 创建考勤信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInfo(@Valid AttendanceInfoSaveReqVO createReqVO);

    /**
     * 更新考勤信息
     *
     * @param updateReqVO 更新信息
     */
    void updateInfo(@Valid AttendanceInfoSaveReqVO updateReqVO);

    /**
     * 删除考勤信息
     *
     * @param id 编号
     */
    void deleteInfo(Long id);

    /**
     * 获得考勤信息
     *
     * @param id 编号
     * @return 考勤信息
     */
    AttendanceInfoDO getInfo(Long id);

    /**
     * 获得考勤信息分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤信息分页
     */
    PageResult<AttendanceInfoDO> getInfoPage(AttendanceInfoPageReqVO pageReqVO);

    /**
     * 批量导入考勤信息
     *
     * @param importAttendanceInfos 导入考勤信息列表
     * @return 导入结果
     */
    String importAttendanceInfoList(List<AttendanceInfoImportExcelVO> importAttendanceInfos);

    /**
     * 获得考勤信息分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤信息分页
     */
    PageResult<AttendanceDailyInfoRespVO> getDailyInfoPage(AttendanceDailyInfoPageReqVO pageReqVO);

    /**
     * 获得考勤信息分页
     *
     * @param pageReqVO 分页查询
     * @return 考勤信息分页
     */
    PageResult<AttendanceMonthlyInfoRespVO> getMonthlyInfoPage(AttendanceMonthlyInfoPageReqVO pageReqVO);

}