package cn.iocoder.yudao.module.attendance.service.leave;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.attendance.api.leave.dto.AttendanceLeaveCreateReqDTO;
import cn.iocoder.yudao.module.attendance.controller.admin.leave.vo.AttendanceLeavePageReqVO;
import cn.iocoder.yudao.module.attendance.controller.admin.leave.vo.AttendanceLeaveSaveReqVO;
import cn.iocoder.yudao.module.attendance.dal.dataobject.leave.AttendanceLeaveDO;

import javax.validation.Valid;
import java.time.LocalDate;
import java.util.List;

/**
 * 请假信息 Service 接口
 *
 * @author zhaopq
 */
public interface AttendanceLeaveService {

    /**
     * 创建请假信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLeave(@Valid AttendanceLeaveSaveReqVO createReqVO);

    /**
     * 审批完成添加请假信息
     *
     * @param leaveCreateReqDTO 请假信息
     * @return 请假信息
     */
    Long createLeaveInfoAfterAudit(AttendanceLeaveCreateReqDTO leaveCreateReqDTO);

    /**
     * 更新请假信息
     *
     * @param updateReqVO 更新信息
     */
    void updateLeave(@Valid AttendanceLeaveSaveReqVO updateReqVO);

    /**
     * 删除请假信息
     *
     * @param id 编号
     */
    void deleteLeave(Long id);

    /**
     * 获得请假信息
     *
     * @param id 编号
     * @return 请假信息
     */
    AttendanceLeaveDO getLeave(Long id);

    /**
     * 获得请假信息分页
     *
     * @param pageReqVO 分页查询
     * @return 请假信息分页
     */
    PageResult<AttendanceLeaveDO> getLeavePage(AttendanceLeavePageReqVO pageReqVO);

    List<AttendanceLeaveDO> selectByDateAndEmployeeId(LocalDate date, Long employeeId);

}