package cn.iocoder.yudao.module.attendance.service.out;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.attendance.api.out.dto.AttendanceOutCreateReqDTO;
import cn.iocoder.yudao.module.attendance.controller.admin.out.vo.AttendanceOutPageReqVO;
import cn.iocoder.yudao.module.attendance.controller.admin.out.vo.AttendanceOutSaveReqVO;
import cn.iocoder.yudao.module.attendance.dal.dataobject.out.AttendanceOutDO;

import javax.validation.Valid;

/**
 * 公务外出信息 Service 接口
 *
 * @author zhaopq
 */
public interface AttendanceOutService {

    /**
     * 创建公务外出信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOut(@Valid AttendanceOutSaveReqVO createReqVO);

    /**
     * 审批完成添加公务外出信息
     *
     * @param outCreateReqDTO 公务外出信息
     * @return 公务外出信息id
     */
    Long createOutInfoAfterAudit(AttendanceOutCreateReqDTO outCreateReqDTO);

    /**
     * 更新公务外出信息
     *
     * @param updateReqVO 更新信息
     */
    void updateOut(@Valid AttendanceOutSaveReqVO updateReqVO);

    /**
     * 删除公务外出信息
     *
     * @param id 编号
     */
    void deleteOut(Long id);

    /**
     * 获得公务外出信息
     *
     * @param id 编号
     * @return 公务外出信息
     */
    AttendanceOutDO getOut(Long id);

    /**
     * 获得公务外出信息分页
     *
     * @param pageReqVO 分页查询
     * @return 公务外出信息分页
     */
    PageResult<AttendanceOutDO> getOutPage(AttendanceOutPageReqVO pageReqVO);

}