package cn.iocoder.yudao.module.employee.controller.admin.info;

import cn.iocoder.yudao.module.employee.controller.admin.info.vo.EmployeeInfoHistoryPageReqVO;
import cn.iocoder.yudao.module.employee.controller.admin.info.vo.EmployeeInfoHistoryRespVO;
import cn.iocoder.yudao.module.employee.controller.admin.info.vo.EmployeeInfoHistorySaveReqVO;
import cn.iocoder.yudao.module.employee.dal.dataobject.info.EmployeeInfoHistoryDO;
import cn.iocoder.yudao.module.employee.service.info.EmployeeInfoHistoryService;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;


@Tag(name = "管理后台 - 员工信息历史")
@RestController
@RequestMapping("/personnel/employee/history")
@Validated
public class EmployeeInfoHistoryController {

    @Resource
    private EmployeeInfoHistoryService infoHistoryService;

    @PostMapping("/create")
    @Operation(summary = "创建员工信息历史")
    @PreAuthorize("@ss.hasPermission('employee:info-history:create')")
    public CommonResult<Long> createInfoHistory(@Valid @RequestBody EmployeeInfoHistorySaveReqVO createReqVO) {
        return success(infoHistoryService.createInfoHistory(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新员工信息历史")
    @PreAuthorize("@ss.hasPermission('employee:info-history:update')")
    public CommonResult<Boolean> updateInfoHistory(@Valid @RequestBody EmployeeInfoHistorySaveReqVO updateReqVO) {
        infoHistoryService.updateInfoHistory(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除员工信息历史")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('employee:info-history:delete')")
    public CommonResult<Boolean> deleteInfoHistory(@RequestParam("id") Long id) {
        infoHistoryService.deleteInfoHistory(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得员工信息历史")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('employee:info:query')")
    public CommonResult<EmployeeInfoHistoryRespVO> getInfoHistory(@RequestParam("id") Long id) {
        EmployeeInfoHistoryDO infoHistory = infoHistoryService.getInfoHistory(id);
        return success(BeanUtils.toBean(infoHistory, EmployeeInfoHistoryRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得员工信息历史分页")
    @PreAuthorize("@ss.hasPermission('employee:info:query')")
    public CommonResult<PageResult<EmployeeInfoHistoryRespVO>> getInfoHistoryPage(@Valid EmployeeInfoHistoryPageReqVO pageReqVO) {
        PageResult<EmployeeInfoHistoryDO> pageResult = infoHistoryService.getInfoHistoryPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, EmployeeInfoHistoryRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出员工信息历史 Excel")
    @PreAuthorize("@ss.hasPermission('employee:info-history:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportInfoHistoryExcel(@Valid EmployeeInfoHistoryPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<EmployeeInfoHistoryDO> list = infoHistoryService.getInfoHistoryPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "员工信息历史.xls", "数据", EmployeeInfoHistoryRespVO.class,
                        BeanUtils.toBean(list, EmployeeInfoHistoryRespVO.class));
    }

}