package cn.iocoder.yudao.module.employee.controller.admin.info.vo;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Schema(description = "管理后台 - 员工信息分页 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class EmployeeInfoQueryReqVO extends PageParam {

    @Schema(description = "员工ID", example = "18657")
    private Long id;

    @Schema(description = "业务UUID", example = "4007")
    private String infoId;

    @Schema(description = "姓名", example = "李四")
    private String name;

    @Schema(description = "部门ID", example = "18657")
    private Long deptId;

    @Schema(description = "部门名称", example = "李四")
    private String deptName;

    @Schema(description = "职位编号", example = "1434")
    private Long postId;

    @Schema(description = "职位名称")
    private String position;

    @Schema(description = "员工编号")
    private String employeeNumber;

    @Schema(description = "入职时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] entryDate;

    @Schema(description = "试用期到期时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] probationEndDate;

    @Schema(description = "离职时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] departureDate;

    @Schema(description = "离职原因", example = "不喜欢")
    private String resignationReason;

    @Schema(description = "员工状态", example = "2")
    private String employeeStatus;

    @Schema(description = "性别")
    private String gender;

    @Schema(description = "电子邮箱")
    private String email;

    @Schema(description = "手机号")
    private String phone;

    @Schema(description = "身份证号")
    private String idCardNumber;

    @Schema(description = "身份证地址")
    private String idCardAddress;

    @Schema(description = "户口类型", example = "1")
    private String householdType;

    @Schema(description = "户口所在地")
    private String householdLocation;

    @Schema(description = "最高学历")
    private String educationLevel;

    @Schema(description = "毕业院校")
    private String graduationSchool;

    @Schema(description = "毕业时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] graduationDate;

    @Schema(description = "银行卡名称", example = "张三")
    private String bankCardName;

    @Schema(description = "银行卡卡号")
    private String bankCardNumber;

    @Schema(description = "年假基准天数")
    private BigDecimal baseAnnualLeave;

    @Schema(description = "已使用年假天数")
    private BigDecimal usedAnnualLeave;

    @Schema(description = "剩余年假天数")
    private BigDecimal remainingAnnualLeave;

    @Schema(description = "出生日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] birthDate;

    @Schema(description = "婚姻状况", example = "1")
    private String marriageStatus;

    @Schema(description = "工作地点")
    private String workLocation;

    @Schema(description = "专业")
    private String major;

    @Schema(description = "参加工作时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDate[] workStartDate;

    @Schema(description = "薪酬")
    private BigDecimal salary;

    @Schema(description = "岗位薪资")
    private BigDecimal positionSalary;

    @Schema(description = "项目津贴")
    private BigDecimal projectAllowance;

    @Schema(description = "特殊岗位津贴")
    private BigDecimal specialPositionAllowance;

    @Schema(description = "外籍津贴")
    private BigDecimal foreignAllowance;

    @Schema(description = "午餐补助")
    private BigDecimal lunchSubsidy;

    @Schema(description = "特别津贴")
    private BigDecimal specialAllowance;

    @Schema(description = "补贴")
    private BigDecimal subsidy;

    @Schema(description = "年终奖")
    private BigDecimal yearEndBonus;

    @Schema(description = "头像地址")
    private String avatar;

    @Schema(description = "状态", example = "2")
    private Integer status;

    @Schema(description = "创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] createTime;

    @Schema(description = "租户编号", example = "10115")
    private Long tenantId;

    @Schema(description = "用户ID", example = "4877")
    private Long userId;

}