package cn.iocoder.yudao.module.employee.controller.admin.info.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import com.alibaba.excel.annotation.*;

@Schema(description = "管理后台 - 员工信息 Response VO")
@Data
@ExcelIgnoreUnannotated
public class EmployeeInfoRespVO {

    @Schema(description = "员工ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "10984")
    private Long id;

    @Schema(description = "业务UUID", requiredMode = Schema.RequiredMode.REQUIRED, example = "1987")
    private String infoId;

    @Schema(description = "姓名", requiredMode = Schema.RequiredMode.REQUIRED, example = "赵六")
    @ExcelProperty("姓名")
    private String name;

    @Schema(description = "部门ID", example = "22378")
    private Long deptId;

    @Schema(description = "部门名称", example = "芋艿")
    @ExcelProperty("部门")
    private String deptName;

    @Schema(description = "职位编号", example = "1056")
    private Long postId;

    @Schema(description = "职位名称")
    @ExcelProperty("职位")
    private String position;

    @Schema(description = "手机号")
    @ExcelProperty("手机号")
    private String phone;

    @Schema(description = "员工编号")
    private String employeeNumber;

    @Schema(description = "入职时间")
    @ExcelProperty("入职时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate entryDate;

    @Schema(description = "试用期到期时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate probationEndDate;

    @Schema(description = "离职时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate departureDate;

    @Schema(description = "离职原因", example = "不香")
    private String resignationReason;

    @Schema(description = "员工状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "2")
    private String employeeStatus;

    @Schema(description = "员工状态", example = "在职")
    @ExcelProperty("员工状态")
    private String employeeStatusDesc;

    @Schema(description = "性别")
    private String gender;

    @Schema(description = "电子邮箱")
    private String email;

    @Schema(description = "身份证号")
    private String idCardNumber;

    @Schema(description = "身份证地址")
    private String idCardAddress;

    @Schema(description = "户口类型", example = "2")
    private String householdType;

    @Schema(description = "户口所在地")
    private String householdLocation;

    @Schema(description = "最高学历")
    private String educationLevel;

    @Schema(description = "毕业院校")
    private String graduationSchool;

    @Schema(description = "毕业时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate graduationDate;

    @Schema(description = "银行卡名称", example = "张三")
    private String bankCardName;

    @Schema(description = "银行卡卡号")
    private String bankCardNumber;

    @Schema(description = "年假基准天数")
    private BigDecimal baseAnnualLeave;

    @Schema(description = "已使用年假天数")
    private BigDecimal usedAnnualLeave;

    @Schema(description = "剩余年假天数")
    @ExcelProperty("当前可用年假")
    private BigDecimal remainingAnnualLeave;

    @Schema(description = "出生日期")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate birthDate;

    @Schema(description = "婚姻状况", example = "2")
    private String marriageStatus;

    @Schema(description = "工作地点")
    private String workLocation;

    @Schema(description = "专业")
    private String major;

    @Schema(description = "参加工作时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate workStartDate;

    @Schema(description = "薪酬")
    private BigDecimal salary;

    @Schema(description = "岗位薪资")
    private BigDecimal positionSalary;

    @Schema(description = "项目津贴")
    private BigDecimal projectAllowance;

    @Schema(description = "特殊岗位津贴")
    private BigDecimal specialPositionAllowance;

    @Schema(description = "外籍津贴")
    private BigDecimal foreignAllowance;

    @Schema(description = "午餐补助")
    private BigDecimal lunchSubsidy;

    @Schema(description = "特别津贴")
    private BigDecimal specialAllowance;

    @Schema(description = "补贴")
    private BigDecimal subsidy;

    @Schema(description = "年终奖")
    private BigDecimal yearEndBonus;

    @Schema(description = "头像地址")
    private String avatar;

    @Schema(description = "状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "2")
    private Integer status;

    @Schema(description = "创建时间", requiredMode = Schema.RequiredMode.REQUIRED)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    @Schema(description = "租户编号", requiredMode = Schema.RequiredMode.REQUIRED, example = "27933")
    private Long tenantId;

    @Schema(description = "用户ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "12259")
    private Long userId;

    @Schema(description = "最新合同开始日期")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate contractStartDate;

    @Schema(description = "最新合同结束日期")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate contractEndDate;

}