package cn.iocoder.yudao.module.employee.service.info;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.employee.controller.admin.info.vo.EmployeeInfoPageReqVO;
import cn.iocoder.yudao.module.employee.controller.admin.info.vo.EmployeeInfoQueryReqVO;
import cn.iocoder.yudao.module.employee.controller.admin.info.vo.EmployeeInfoSaveReqVO;
import cn.iocoder.yudao.module.employee.dal.dataobject.info.EmployeeInfoDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 员工信息 Service 接口
 *
 * @author zhaopq
 */
public interface EmployeeInfoService {

    /**
     * 获得员工信息列表
     *
     * @param ids 编号集合
     * @return 员工信息列表
     */
    List<EmployeeInfoDO> getEmployeeListByIds(Collection<Long> ids);

    /**
     * 根据用户id获取对应员工信息
     *
     * @param userId 用户id
     * @return 员工信息
     */
    EmployeeInfoDO getEmployeeByUserId(Long userId);

    /**
     * 根据用户id列表，获得员工信息列表
     *
     * @param userIds 用户id列表
     * @return 员工信息列表
     */
    List<EmployeeInfoDO> getInfoListByUserIds(Collection<Long> userIds);


    /**
     * 创建员工信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInfo(@Valid EmployeeInfoSaveReqVO createReqVO);

    /**
     * 更新员工信息（接口用）
     *
     * @param updateReqVO 更新信息
     */
    Integer updateInfo(@Valid EmployeeInfoSaveReqVO updateReqVO);

    /**
     * 更新员工信息（修改用）
     *
     * @param updateReqVO 更新信息
     */
    Integer updateEmpInfo(@Valid EmployeeInfoSaveReqVO updateReqVO);

    /**
     * 删除员工信息
     *
     * @param id 编号
     */
    void deleteInfo(Long id);

    /**
     * 获得员工信息
     *
     * @param id 编号
     * @return 员工信息
     */
    EmployeeInfoDO getInfo(Long id);

    /**
     * 获取当前登录的员工信息
     *
     * @return 当前登录的员工信息对象，包括员工ID、姓名、部门等详细信息
     * @see EmployeeInfoDO 包含员工详细信息的数据对象
     */
    EmployeeInfoDO getLoginEmployee();

    /**
     * 获得员工信息分页
     *
     * @param pageReqVO 分页查询
     * @return 员工信息分页
     */
    PageResult<EmployeeInfoDO> getInfoPage(EmployeeInfoPageReqVO pageReqVO);

    /**
     * 获得可选择的员工信息列表，用于选择业务人，审批人等
     *
     * @return 可选择的员工信息列表
     */
    List<EmployeeInfoDO> getInfoListForSelectEmployee(String auth, String name, Long deptId, String phone);

    /**
     *
     *
     * @param userId
     * @param tenantId
     * @return
     */
    EmployeeInfoDO getInfoByUserIdAndTenantId(Long userId, Long tenantId);

    /**
     * 获得员工信息
     *
     * @param queryReqVO
     * @return
     */
    EmployeeInfoDO getInfo(@Valid EmployeeInfoQueryReqVO queryReqVO);


}