package cn.iocoder.yudao.module.holiday.controller.admin.annualleave;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.employee.api.EmployeeApi;
import cn.iocoder.yudao.module.employee.api.dto.EmployeePageReqDTO;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeQueryReqDTO;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import javax.annotation.Resource;
import javax.validation.Valid;


@Tag(name = "管理后台 - 员工假务信息")
@RestController
@RequestMapping("/personnel/holiday/annual-leave")
@Validated
public class HolidayAnnualLeaveController {

    @Resource
    private EmployeeApi employeeApi;


    @GetMapping("/get")
    @Operation(summary = "获得员工年假信息")
    @PreAuthorize("@ss.hasPermission('holiday:annual-leave:query')")
    public CommonResult<EmployeeRespDTO> get(@Valid EmployeeQueryReqDTO reqDTO) {
        EmployeeRespDTO info = employeeApi.getEmployee(reqDTO);
        return success(info);
    }

    @GetMapping("/page")
    @Operation(summary = "获得员工年假信息分页")
    @PreAuthorize("@ss.hasPermission('holiday:annual-leave:query')")
    public CommonResult<PageResult<EmployeeRespDTO>> page(@Valid EmployeePageReqDTO pageReqVO) {
        PageResult<EmployeeRespDTO> pageResult = employeeApi.getEmployeePage(pageReqVO);
        return success(pageResult);
    }

}