package cn.iocoder.yudao.module.relations.service.renew;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.relations.api.renew.dto.RelationsRenewCreateReqDTO;
import cn.iocoder.yudao.module.relations.controller.admin.renew.vo.RelationsRenewPageReqVO;
import cn.iocoder.yudao.module.relations.controller.admin.renew.vo.RelationsRenewRespVO;
import cn.iocoder.yudao.module.relations.controller.admin.renew.vo.RelationsRenewSaveReqVO;
import cn.iocoder.yudao.module.relations.dal.dataobject.renew.RelationsRenewDO;

import javax.validation.Valid;

/**
 * 续签信息 Service 接口
 *
 * @author zhaopq
 */
public interface RelationsRenewService {

    /**
     * 创建续签信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createRenew(@Valid RelationsRenewSaveReqVO createReqVO);

    /**
     * 审批完成添加合同续签信息
     *
     * @param renewCreateReqDTO 续签信息
     * @return 续签信息id
     */
    Long createRenewInfoAfterAudit(RelationsRenewCreateReqDTO renewCreateReqDTO);

    /**
     * 更新续签信息
     *
     * @param updateReqVO 更新信息
     */
    void updateRenew(@Valid RelationsRenewSaveReqVO updateReqVO);

    /**
     * 删除续签信息
     *
     * @param id 编号
     */
    void deleteRenew(Long id);

    /**
     * 获得续签信息
     *
     * @param id 编号
     * @return 续签信息
     */
    RelationsRenewDO getRenew(Long id);

    /**
     * 通过ID获得续签信息
     *
     * @param id
     * @return
     */
    RelationsRenewRespVO getById(Long id);

    /**
     * 获得续签信息分页
     *
     * @param pageReqVO 分页查询
     * @return 续签信息分页
     */
    PageResult<RelationsRenewDO> getRenewPage(RelationsRenewPageReqVO pageReqVO);

}