package cn.iocoder.yudao.module.system.api.dept;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.module.system.api.dept.dto.PostRespDTO;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 岗位 API 接口
 *
 * @author 芋道源码
 */
public interface PostApi {

    /**
     * 校验岗位们是否有效。如下情况，视为无效：
     * 1. 岗位编号不存在
     * 2. 岗位被禁用
     *
     * @param ids 岗位编号数组
     */
    void validPostList(Collection<Long> ids);

    List<PostRespDTO> getPostList(Collection<Long> ids);

    default Map<Long, PostRespDTO> getPostMap(Collection<Long> ids) {
        if (CollUtil.isEmpty(ids)) {
            return MapUtil.empty();
        }

        List<PostRespDTO> list = getPostList(ids);
        return CollectionUtils.convertMap(list, PostRespDTO::getId);
    }
    
    /**
     * 根据职位ID获取职位详情
     * 此方法用于从数据库中检索特定职位的详细信息，并将其封装到一个响应DTO中
     * 主要用于满足用户查看特定职位的需求
     * 
     * @param id 职位的唯一标识符通过这个ID可以在数据库中找到对应的职位
     * @return 返回一个包含职位详细信息的响应DTO如果找不到对应的职位，则可能返回null
     */
    PostRespDTO getPost(Long id);

}
