package cn.iocoder.yudao.module.system.enums.dicttenant;

import cn.iocoder.yudao.module.system.api.dict.dto.DictDataRespDTO;
import cn.iocoder.yudao.module.system.api.dicttenant.dto.DictDataTenantRespDTO;

import java.util.ArrayList;
import java.util.List;

/**
 * 租户字典类型枚举
 */
public enum DictTypeTenantEnum {

    TENANT_LEAVE_TYPE("tenant_leave_type", "请假类型"),
    TENANT_EMPLOYEE_TYPE("tenant_employee_type", "员工类型"),
    TENANT_TURNOVER_REASON("tenant_turnover_reason", "离职原因"),
    ;

    private final String type;
    private final String label;

    DictTypeTenantEnum(String type, String label) {
        this.type = type;
        this.label = label;
    }

    public String getType() {
        return type;
    }

    public String getLabel() {
        return label;
    }

    /**
     * 根据类型，获得字典标签
     */
    public static String getLabel(String type) {
        for (DictTypeTenantEnum dictTypeEnum : values()) {
            if (dictTypeEnum.getType().equals(type)) {
                return dictTypeEnum.getLabel();
            }
        }
        return null;
    }

    /**
     * 根据类型，获得字典数据
     *
     * @param dictType 字典类型
     * @return 字典数据
     */
    public static List<DictDataTenantRespDTO> getDictData(String dictType) {
        List<DictDataTenantRespDTO> dictDataList = new ArrayList<>();
        if (DictTypeTenantEnum.TENANT_LEAVE_TYPE.getType().equals(dictType)) {
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("事假").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("年假").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("病假").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("婚假").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("产假").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("陪产假").build());
        } else if (DictTypeTenantEnum.TENANT_EMPLOYEE_TYPE.getType().equals(dictType)) {
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("实习生").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("全职").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("兼职").build());
        } else if (DictTypeTenantEnum.TENANT_TURNOVER_REASON.getType().equals(dictType)) {
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("家庭原因").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("健康原因").build());
            dictDataList.add(DictDataTenantRespDTO.builder().dictType(dictType).label("职业规划").build());
        }

        return dictDataList;
    }
}
