package cn.iocoder.yudao.module.system.controller.admin.dictTenant;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.enums.TenantDataTypeEnum;
import cn.iocoder.yudao.module.system.controller.admin.dict.vo.data.DictDataSimpleRespVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantRespVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantSaveReqVO;

import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantUpdateReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.dictTenant.DictDataTenantDO;
import cn.iocoder.yudao.module.system.service.dictTenant.DictDataTenantService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import org.springdoc.api.annotations.ParameterObject;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;



@Tag(name = "管理后台 - 租户字典数据")
@RestController
@RequestMapping("/system/tenant/dict-data")
@Validated
public class DictDataTenantController {

    @Resource
    private DictDataTenantService dictDataTenantService;

    @PostMapping("/create")
    @Operation(summary = "新增租户字典数据")
    @ApiOperationSupport(order = 1)
    // @PreAuthorize("@ss.hasPermission('tenant:dict-data:create')")
    public CommonResult<Long> createDictData(@Valid @RequestBody DictDataTenantSaveReqVO createReqVO) {
        return success(dictDataTenantService.createDictData(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "修改租户字典数据")
    @ApiOperationSupport(order = 2)
    // @PreAuthorize("@ss.hasPermission('tenant:dict-data:update')")
    public CommonResult<Boolean> updateDictData(@Valid @RequestBody DictDataTenantUpdateReqVO updateReqVO) {
        dictDataTenantService.updateDictData(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除租户字典数据")
    @ApiOperationSupport(order = 3)
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('tenant:dict-data:delete')")
    public CommonResult<Boolean> deleteDictData(@RequestParam("id") Long id) {
        dictDataTenantService.deleteDictData(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得租户字典数据")
    @ApiOperationSupport(order = 4)
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    // @PreAuthorize("@ss.hasPermission('tenant:dict-data:query')")
    public CommonResult<DictDataTenantRespVO> getDictData(@RequestParam("id") Long id) {
        DictDataTenantDO dictData = dictDataTenantService.getDictData(id);
        return success(BeanUtils.toBean(dictData, DictDataTenantRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得租户字典数据分页")
    @ApiOperationSupport(order = 5)
    // @PreAuthorize("@ss.hasPermission('tenant:dict-data:query')")
    public CommonResult<PageResult<DictDataTenantRespVO>> getDictDataPage(@Valid @ParameterObject DictDataTenantPageReqVO pageReqVO) {
        PageResult<DictDataTenantDO> pageResult = dictDataTenantService.getDictDataPage(pageReqVO);
        PageResult<DictDataTenantRespVO> voPageResult = BeanUtils.toBean(pageResult, DictDataTenantRespVO.class);
        List<DictDataTenantRespVO> list = voPageResult.getList();
        if (CollUtil.isNotEmpty(list)) {
            Map<Integer, String> statusMap = Arrays.stream(CommonStatusEnum.values())
                    .collect(Collectors.toMap(CommonStatusEnum::getStatus, CommonStatusEnum::getName));

            Map<Integer, String> dataTypeMap = Arrays.stream(TenantDataTypeEnum.values())
                    .collect(Collectors.toMap(TenantDataTypeEnum::getType, TenantDataTypeEnum::getName));

            for (DictDataTenantRespVO dictData : list) {
                dictData.setStatusDesc(statusMap.get(dictData.getStatus()));
                dictData.setDataTypeDesc(dataTypeMap.get(dictData.getDataType()));
            }
        }
        return success(voPageResult);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出租户字典数据 Excel")
    @ApiOperationSupport(order = 6)
    // @PreAuthorize("@ss.hasPermission('tenant:dict-data:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportDictDataExcel(@Valid @ParameterObject DictDataTenantPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<DictDataTenantDO> list = dictDataTenantService.getDictDataPage(pageReqVO).getList();
        List<DictDataTenantRespVO> voList = BeanUtils.toBean(list, DictDataTenantRespVO.class);
        if (CollUtil.isNotEmpty(voList)) {
            Map<Integer, String> statusMap = Arrays.stream(CommonStatusEnum.values())
                    .collect(Collectors.toMap(CommonStatusEnum::getStatus, CommonStatusEnum::getName));

            Map<Integer, String> dataTypeMap = Arrays.stream(TenantDataTypeEnum.values())
                    .collect(Collectors.toMap(TenantDataTypeEnum::getType, TenantDataTypeEnum::getName));

            for (DictDataTenantRespVO dictData : voList) {
                dictData.setStatusDesc(statusMap.get(dictData.getStatus()));
                dictData.setDataTypeDesc(dataTypeMap.get(dictData.getDataType()));
            }
        }

        // 导出 Excel
        ExcelUtils.write(response, "租户字典数据.xls", "数据", DictDataTenantRespVO.class,
                        voList);
    }

    @GetMapping(value = {"/list-all-simple"})
    @Operation(summary = "根据字典类型获取所有开启状态的数据列表", description = "用于新增和修改业务")
    @ApiOperationSupport(order = 7)
    @Parameter(name = "dictType", description = "字典类型", required = true)
    // 无需添加权限认证，因为前端全局都需要
    public CommonResult<List<DictDataSimpleRespVO>> getSimpleDictDataList(@RequestParam("dictType") String dictType) {
        List<DictDataTenantDO> list = dictDataTenantService.getDictDataList(
                CommonStatusEnum.ENABLE.getStatus(), dictType);
        return success(BeanUtils.toBean(list, DictDataSimpleRespVO.class));
    }

    @GetMapping(value = {"/list-all"})
    @Operation(summary = "根据字典类型获取所有状态的数据列表", description = "用于查询业务")
    @ApiOperationSupport(order = 8)
    @Parameter(name = "dictType", description = "字典类型", required = true)
    // 无需添加权限认证，因为前端全局都需要
    public CommonResult<List<DictDataSimpleRespVO>> getAllDictDataList(@RequestParam("dictType") String dictType) {
        List<DictDataTenantDO> list = dictDataTenantService.getDictDataList(
                null, dictType);
        return success(BeanUtils.toBean(list, DictDataSimpleRespVO.class));
    }

}